import os
import csv
import json
import cmind


def main():
    f = os.environ.get('CM_PROCESS_AE_USERS_INPUT_FILE', '')

    print('Input CSV file: {}'.format(f))

    users = []
    with open(f, 'r') as ff:
        csvreader = csv.DictReader(ff)
        for row in csvreader:
            if len(row) > 0:
                users.append(row)

    print('')
    html = '<ul>\n'
    for user in sorted(users, key=lambda u: (
            u['last'].lower(), u['first'].lower())):

        full_name = user['first'] + ' ' + user['last']

        name = full_name + ' (' + user['affiliation'] + ')'

        print(name)

        html += '  <li>' + name + '\n'

        # Checking contributor
        r = cmind.access({'action': 'find',
                          'automation': 'contributor',
                          'artifact': full_name})
        if r['return'] > 0:
            return r

        lst = r['list']

        if len(lst) == 0:
            print('  CM contributor not found!')

            meta = {
                'challenges': [
                    'ae-micro2023'
                ],
                'last_participation_date': '202309',
                'name': full_name,
                'organization': user['affiliation']
            }

            print('  Adding to mlcommons@ck ...')
            r = cmind.access({'out': 'con',
                              'action': 'add',
                              # Need UID since using common function
                              'automation': 'contributor,68eae17b590d4f8f',
                              'artifact': 'mlcommons@ck:' + full_name,
                              'meta': meta,
                              'common': True
                              })
            if r['return'] > 0:
                return r

    html += '</ul>\n'

    fo = f + '.html'

    print('')
    print('Saved HTML to {}'.format(fo))

    cmind.utils.save_txt(fo, html)

    return {'return': 0}


if __name__ == '__main__':
    r = main()
    if r['return'] > 0:
        cmind.error(r)



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    automation = i['automation']
    cm = automation.cmind

    # If windows, download here otherwise use run.sh
    if os_info['platform'] == 'windows':

        path = os.getcwd()

        clean_dirs = env.get('CM_CLEAN_DIRS', '').strip()
        if clean_dirs != '':
            import shutil
            for cd in clean_dirs.split(','):
                if cd != '':
                    if os.path.isdir(cd):
                        print('Clearning directory {}'.format(cd))
                        shutil.rmtree(cd)

        url = env['CM_PACKAGE_WIN_URL']

        urls = [url] if ';' not in url else url.split(';')

        print('')
        print('Current directory: {}'.format(os.getcwd()))

        for url in urls:

            url = url.strip()

            print('')
            print('Downloading from {}'.format(url))

            r = cm.access({'action': 'download_file',
                           'automation': 'utils,dc2743f8450541e3',
                           'url': url})
            if r['return'] > 0:
                return r

            filename = r['filename']

            print('Unzipping file {}'.format(filename))

            r = cm.access({'action': 'unzip_file',
                           'automation': 'utils,dc2743f8450541e3',
                           'filename': filename})
            if r['return'] > 0:
                return r

            if os.path.isfile(filename):
                print('Removing file {}'.format(filename))
                os.remove(filename)

        print('')

        # Add to path
        env['+PATH'] = [os.path.join(path, 'bin')]

    return {'return': 0}



# This test covers version, variation, compilation from src,
# add_deps_recursive, post_deps

import cmind as cm
import check as checks

r = cm.access({'action': 'run',
               'automation': 'script',
               'tags': 'run,docker,container',
               'add_deps_recursive': {
                   'compiler': {'tags': "gcc"}
               },
               'docker_cm_repo': 'mlcommons@mlperf-automations',
               'image_name': 'cm-script-app-image-classification-onnx-py',
               'env': {
                   'CM_DOCKER_RUN_SCRIPT_TAGS': 'app,image-classification,onnx,python',
                   'CM_DOCKER_IMAGE_BASE': 'ubuntu:22.04',
                   'CM_DOCKER_IMAGE_REPO': 'cknowledge'
               },
               'quiet': 'yes'
               })
checks.check_return(r)

r = cm.access({'action': 'run',
               'automation': 'script',
               'tags': 'run,docker,container',
               'add_deps_recursive': {
                   'compiler': {'tags': "gcc"}
               },
               'docker_cm_repo': 'mlcommons@mlperf-automations',
               'image_name': 'cm-script-app-image-classification-onnx-py',
               'env': {
                   'CM_DOCKER_RUN_SCRIPT_TAGS': 'app,image-classification,onnx,python',
                   'CM_DOCKER_IMAGE_BASE': 'ubuntu:24.04',
                   'CM_DOCKER_IMAGE_REPO': 'local'
               },
               'quiet': 'yes'
               })
checks.check_return(r)



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):
    os_info = i['os_info']
    env = i['env']

    machine = env.get('CM_HOST_OS_MACHINE', '')
    if machine == '':
        machine = 'x86_64'
    if machine == 'x86_64':
        machine = 'x64'

    hostos = env['CM_HOST_OS_TYPE']

    ext = '.tgz'

    if hostos == 'darwin':
        hostos = 'osx'
    elif hostos == 'windows':
        hostos = 'win'
        ext = '.zip'

    device = env.get('CM_ONNXRUNTIME_DEVICE', '')
    if device != '':
        machine += '-' + device

    version = env['CM_VERSION']

    FOLDER = 'onnxruntime-{}-{}-{}'.format(hostos, machine, version)

    FILENAME = FOLDER + ext

    URL = 'https://github.com/microsoft/onnxruntime/releases/download/v{}/{}'.format(
        version, FILENAME)

    print('')
    print('Downloading from {}'.format(URL))
    print('')

    env['FOLDER'] = FOLDER
    env['FILENAME'] = FILENAME
    env['URL'] = URL

    return {'return': 0}


def postprocess(i):

    env = i['env']

    hostos = env['CM_HOST_OS_TYPE']

    install_folder = env['CM_TMP_INSTALL_FOLDER']

    for key in ['+C_INCLUDE_PATH', '+CPLUS_INCLUDE_PATH',
                '+LD_LIBRARY_PATH', '+DYLD_FALLBACK_LIBRARY_PATH']:
        #        20221024: we save and restore env in the main script and can clean env here for determinism
        #        if key not in env:
        env[key] = []

    include_path = os.path.join(
        os.getcwd(),
        'install',
        install_folder,
        'include')

    env['+C_INCLUDE_PATH'].append(include_path)
    env['+CPLUS_INCLUDE_PATH'].append(include_path)

    lib_path = os.path.join(os.getcwd(), 'install', install_folder, 'lib')

    env['+LD_LIBRARY_PATH'].append(lib_path)
    env['+DYLD_FALLBACK_LIBRARY_PATH'].append(lib_path)

    if hostos == 'windows':
        # For dynamic libraries
        env['+PATH'] = [lib_path]

    env['CM_ONNXRUNTIME_LIB_PATH'] = lib_path
    env['CM_ONNXRUNTIME_INCLUDE_PATH'] = include_path

    return {'return': 0}



import os
import sys

package_name = os.environ.get('CM_GENERIC_PYTHON_PACKAGE_NAME', '')
package_name = package_name.split("[")[0]

filename = 'tmp-ver.out'

if os.path.isfile(filename):
    os.remove(filename)

if package_name != '':

    version = ''
    error = ''

    try:
        import importlib.metadata
        version = importlib.metadata.version(package_name)
    except Exception as e:
        error = format(e)

    if error != '' and sys.version_info < (3, 9):
        try:
            import pkg_resources
            version = pkg_resources.get_distribution(package_name).version
            error = ''
        except Exception as e:
            if error != '':
                error += '\n'
            error += format(e)

    # We generally skip error since it usually means that
    # package is not installed

    with open(filename, 'w') as file:
        file.write(str(version) + '\n')



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']
    platform = os_info['platform']

    env = i['env']

    automation = i['automation']

    recursion_spaces = i['recursion_spaces']

    run_script_input = i['run_script_input']

    # Check if ANDROID_HOME is already set
    android_home = os.environ.get('ANDROID_HOME', '').strip()

    # We are inside CM cache entry
    cur_dir = os.getcwd()

    if android_home == '':
        android_home = cur_dir

    env['CM_ANDROID_HOME'] = android_home
    env['ANDROID_HOME'] = android_home

    paths = []

    # Check SDK manager
    ext = ''
    host_os_for_android = 'linux'
    host_os_for_ndk = 'linux-x86_64'
    if platform == "windows":
        host_os_for_android = 'win'
        host_os_for_ndk = 'windows-x86_64'
        ext = '.bat'
    elif platform == "darwin":
        host_os_for_android = 'mac'

    sdk_manager_file = 'sdkmanager' + ext

    print('')

    found = False

    for x in ['cmdline-tools', 'cmdline-tools' + os.sep + 'tools', 'tools']:
        sdk_manager_path = os.path.join(
            android_home, x, 'bin', sdk_manager_file)
        if os.path.isfile(sdk_manager_path):
            found = True
            break

    if not found:
        # Some magic for cmdline tools (need specific directory)
        new_path = os.path.join(android_home, 'cmdline-tools')
        if not os.path.isdir(new_path):
            os.makedirs(new_path)

        os.chdir(new_path)

        cmdline_tools_version = env.get('CM_ANDROID_CMDLINE_TOOLS_VERSION', '')

        env['CM_ANDROID_CMDLINE_TOOLS_VERSION'] = cmdline_tools_version

        package_url = env['CM_ANDROID_CMDLINE_TOOLS_URL']
        package_url = package_url.replace(
            '${CM_ANDROID_CMDLINE_TOOLS_OS}',
            host_os_for_android)
        package_url = package_url.replace(
            '${CM_ANDROID_CMDLINE_TOOLS_VERSION}',
            cmdline_tools_version)

        env['CM_ANDROID_CMDLINE_TOOLS_URL'] = package_url

        print('')
        print('Downloading from {} ...'.format(package_url))

        cm = automation.cmind

        r = cm.access({'action': 'download_file',
                       'automation': 'utils,dc2743f8450541e3',
                       'url': package_url})
        if r['return'] > 0:
            return r

        filename = r['filename']

        print('Unzipping file {}'.format(filename))

        r = cm.access({'action': 'unzip_file',
                       'automation': 'utils,dc2743f8450541e3',
                       'filename': filename,
                       'strip_folders': 0})
        if r['return'] > 0:
            return r

#        if os.path.isfile(filename):
#            print ('Removing file {}'.format(filename))
#            os.remove(filename)

        os.rename('cmdline-tools', 'tools')

        os.chdir(cur_dir)

        sdk_manager_path = os.path.join(
            android_home,
            'cmdline-tools',
            'tools',
            'bin',
            sdk_manager_file)

    sdk_manager_dir = os.path.dirname(sdk_manager_path)

    env['CM_ANDROID_SDK_MANAGER_BIN'] = sdk_manager_file
    env['CM_ANDROID_SDK_MANAGER_BIN_WITH_PATH'] = sdk_manager_path

    env['CM_GET_DEPENDENT_CACHED_PATH'] = cur_dir

    paths.append(sdk_manager_dir)

    # Prepare SDK
    print('Preparing Android SDK manager ...')

    r = automation.run_native_script(
        {'run_script_input': run_script_input, 'env': env, 'script_name': 'prepare-sdk-manager'})
    if r['return'] > 0:
        return r

    build_tools_version = env['CM_ANDROID_BUILD_TOOLS_VERSION']

    path_build_tools = os.path.join(
        android_home, 'build-tools', build_tools_version)
    env['CM_ANDROID_BUILD_TOOLS_PATH'] = path_build_tools
    paths.append(path_build_tools)

    cmake_version = env['CM_ANDROID_CMAKE_VERSION']

    path_cmake = os.path.join(android_home, 'cmake', cmake_version, 'bin')
    env['CM_ANDROID_CMAKE_PATH'] = path_cmake
    paths.append(path_cmake)

    path_emulator = os.path.join(android_home, 'emulator')
    env['CM_ANDROID_EMULATOR_PATH'] = path_emulator
    paths.append(path_emulator)

    path_platform_tools = os.path.join(android_home, 'platform-tools')
    env['CM_ANDROID_PLATFORM_TOOLS_PATH'] = path_platform_tools
    paths.append(path_platform_tools)

    android_version = env['CM_ANDROID_VERSION']

    path_platforms = os.path.join(android_home, 'platforms', android_version)
    env['CM_ANDROID_PLATFORMS_PATH'] = path_platforms

    path_tools = os.path.join(android_home, 'tools')
    env['CM_ANDROID_TOOLS_PATH'] = path_tools
    paths.append(path_tools)

    android_ndk_version = env['CM_ANDROID_NDK_VERSION']

    # Check Android NDK
    path_ndk = os.path.join(android_home, 'ndk', android_ndk_version)
    env['CM_ANDROID_NDK_PATH'] = path_ndk
    env['ANDROID_NDK_HOME'] = path_ndk

    path_ndk_compiler = os.path.join(
        path_ndk,
        'toolchains',
        'llvm',
        'prebuilt',
        host_os_for_ndk,
        'bin')
    env['CM_ANDROID_LLVM_PATH'] = path_ndk_compiler
    env['CM_ANDROID_LLVM_CLANG_BIN_WITH_PATH'] = os.path.join(
        path_ndk_compiler, 'clang.exe')
    paths.append(path_ndk_compiler)

    env['+PATH'] = paths

    return {'return': 0}  # , 'version': version}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import re


def preprocess(i):

    os_info = i['os_info']

    env = i['env']
    state = i['state']
    automation = i['automation']

    # Use VERSION_CMD and CHECK_CMD if no CHECK_CMD is set
    if env.get('CM_SYS_UTIL_VERSION_CMD', '') != '' and env.get(
            'CM_SYS_UTIL_CHECK_CMD', '') == '':
        env['CM_SYS_UTIL_CHECK_CMD'] = env['CM_SYS_UTIL_VERSION_CMD']

    if env.get('CM_GENERIC_SYS_UTIL_RUN_MODE', '') == "install":
        if env.get('CM_SYS_UTIL_INSTALL_WITH_RETRY', '') == "yes":
            i['run_script_input']['script_name'] = "install-with-retry"
        else:
            i['run_script_input']['script_name'] = "install"

    if env.get('CM_GENERIC_SYS_UTIL_RUN_MODE', '') == "detect":
        if env.get('CM_SYS_UTIL_VERSION_CMD', '') != '' or env.get(
                'CM_SYS_UTIL_VERSION_CMD_OVERRIDE', '') != '':
            r = automation.run_native_script(
                {'run_script_input': i['run_script_input'], 'env': env, 'script_name': 'detect'})
            if r['return'] != 0:  # detection failed, do install via prehook_deps
                print("detection failed, going for installation")
                env['CM_GENERIC_SYS_UTIL_INSTALL_NEEDED'] = "yes"
                return {'return': 0}
            else:  # detection is successful, no need to install
                # print("detection success")
                env['CM_SYS_UTIL_INSTALL_CMD'] = ""
                return {'return': 0}
        else:  # No detction command available, just install
            # print("No detection possible, going for installation")
            env['CM_GENERIC_SYS_UTIL_INSTALL_NEEDED'] = "yes"
            return {'return': 0}

    # Only "install" mode reaches here
    pm = env.get('CM_HOST_OS_PACKAGE_MANAGER')
    util = env.get('CM_SYS_UTIL_NAME', '')
    if util == '':
        return {
            'return': 1, 'error': 'Please select a variation specifying the sys util name'}

    package = state.get(util)
    package_name = None
    if package and pm:
        package_name = package.get(pm)

    if os_info['platform'] == 'windows' and not package_name:
        print('')
        print('WARNING: for now skipping get-generic-sys-util on Windows ...')
        print('')

        return {'return': 0}

    if not pm:
        return {'return': 1, 'error': 'Package manager not detected for the given OS'}

    if not package:
        return {'return': 1,
                'error': f'No package name specified for {util} in the meta'}

    if not package_name:
        if str(env.get('CM_GENERIC_SYS_UTIL_IGNORE_MISSING_PACKAGE', '')
               ).lower() in ["1", "true", "yes"]:
            print(
                f"WARNING: No package name specified for {pm} and util name {util}. Ignoring it...")
            env['CM_TMP_GENERIC_SYS_UTIL_PACKAGE_INSTALL_IGNORED'] = 'yes'
            return {'return': 0}
        else:
            return {
                'return': 1, 'error': f'No package name specified for {pm} and util name {util}'}

    if util == "libffi":
        if env.get("CM_HOST_OS_FLAVOR", "") == "ubuntu":
            if env.get("CM_HOST_OS_VERSION", "") in [
                    "20.04", "20.10", "21.04", "21.10"]:
                package_name = "libffi7"
            else:
                package_name = "libffi8"

    # Temporary handling of dynamic state variables
    tmp_values = re.findall(r'<<<(.*?)>>>', str(package_name))
    for tmp_value in tmp_values:
        if tmp_value not in env:
            return {'return': 1,
                    'error': 'variable {} is not in env'.format(tmp_value)}
        if tmp_value in env:
            if isinstance(package_name, str):
                package_name = package_name.replace(
                    "<<<" + tmp_value + ">>>", str(env[tmp_value]))

    install_cmd = env.get('CM_HOST_OS_PACKAGE_MANAGER_INSTALL_CMD')
    if not install_cmd:
        return {
            'return': 1, 'error': 'Package manager installation command not detected for the given OS'}

    if pm == "brew":
        sudo = ''
    else:
        sudo = env.get('CM_SUDO', '')
    env['CM_SYS_UTIL_INSTALL_CMD'] = sudo + \
        ' ' + install_cmd + ' ' + package_name

    env['+PATH'] = []

    if env.get('CM_HOST_OS_FLAVOR', '') == 'rhel':
        if env['CM_SYS_UTIL_NAME'] == "g++12":
            env['+PATH'] = ["/opt/rh/gcc-toolset-12/root/usr/bin"]

        if env['CM_SYS_UTIL_NAME'] == "numactl" and env['CM_HOST_OS_VERSION'] in [
                "9.1", "9.2", "9.3"]:
            env['CM_SYS_UTIL_INSTALL_CMD'] = ''

    if env.get('CM_SYS_UTIL_CHECK_CMD',
               '') != '' and env['CM_SYS_UTIL_INSTALL_CMD'] != '':
        env['CM_SYS_UTIL_INSTALL_CMD'] = f"""{env['CM_SYS_UTIL_CHECK_CMD']} || {env['CM_SYS_UTIL_INSTALL_CMD']}"""

    return {'return': 0}


def detect_version(i):
    env = i['env']
    version_env_key = f"CM_{env['CM_SYS_UTIL_NAME'].upper()}_VERSION"
    version_check_re = env.get('CM_SYS_UTIL_VERSION_RE', '')
    group_number = env.get('CM_TMP_VERSION_DETECT_GROUP_NUMBER', 1)

    # Confirm that the regex pattern and file are present
    if version_check_re == '' or not os.path.exists("tmp-ver.out"):
        version = "undetected"
    else:
        r = i['automation'].parse_version({'match_text': version_check_re,
                                           'group_number': group_number,
                                           'env_key': version_env_key,
                                           'which_env': env})

        if r['return'] > 0:
            return r

        version = r['version']
        print(
            i['recursion_spaces'] +
            '    Detected version: {}'.format(version))

    return {'return': 0, 'version': version}


def postprocess(i):
    env = i['env']

    version_env_key = f"CM_{env['CM_SYS_UTIL_NAME'].upper()}_VERSION"

    if (env.get('CM_SYS_UTIL_VERSION_CMD', '') != '' or env.get('CM_SYS_UTIL_VERSION_CMD_OVERRIDE', '') != '') and env.get(version_env_key, '') == '' and str(env.get(
            'CM_TMP_GENERIC_SYS_UTIL_PACKAGE_INSTALL_IGNORED', '')).lower() not in ["yes", "1", "true"] and env.get('CM_GET_GENERIC_SYS_UTIL_INSTALL_FAILED', '') != 'yes':
        automation = i['automation']

        r = automation.run_native_script(
            {'run_script_input': i['run_script_input'], 'env': env, 'script_name': 'detect'})
        if r['return'] > 0 and str(env.get(
                'CM_GENERIC_SYS_UTIL_IGNORE_VERSION_DETECTION_FAILURE', '')).lower() not in ["1", "yes", "true"]:
            return {'return': 1, 'error': 'Version detection failed after installation. Please check the provided version command or use env.CM_GENERIC_SYS_UTIL_IGNORE_VERSION_DETECTION_FAILURE=yes to ignore the error.'}

        elif r['return'] == 0:
            r = detect_version(i)

            if r['return'] > 0:
                return r

            version = r['version']

            env[version_env_key] = version

            # Not used now
            env['CM_GENERIC_SYS_UTIL_' + env['CM_SYS_UTIL_NAME'].upper() +
                '_CACHE_TAGS'] = 'version-' + version

    if env.get(version_env_key, '') == '':
        env[version_env_key] = "undetected"

    return {'return': 0, 'version': env[version_env_key]}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    env['CM_GIT_CHECKOUT'] = "v" + env['CM_VERSION']
    quiet = (env.get('CM_QUIET', False) == 'yes')

    return {'return': 0}


def postprocess(i):

    env = i['env']
    env['+C_INCLUDE_PATH'] = []
    env['+CPLUS_INCLUDE_PATH'] = []
    env['+LD_LIBRARY_PATH'] = []

    protobuf_install_path = os.path.join(os.getcwd(), "install")
    env['CM_GOOGLE_PROTOBUF_SRC_PATH'] = env['CM_GIT_REPO_CHECKOUT_PATH']
    env['CM_GOOGLE_PROTOBUF_INSTALL_PATH'] = protobuf_install_path
    env['+C_INCLUDE_PATH'].append(
        os.path.join(
            protobuf_install_path,
            "include"))
    env['+CPLUS_INCLUDE_PATH'].append(
        os.path.join(protobuf_install_path, "include"))

    if os.path.exists(os.path.join(protobuf_install_path, "lib")):
        env['+LD_LIBRARY_PATH'].append(
            os.path.join(protobuf_install_path, "lib"))
    elif os.path.exists(os.path.join(protobuf_install_path, "lib64")):
        env['+LD_LIBRARY_PATH'].append(
            os.path.join(protobuf_install_path, "lib64"))
    else:
        return {
            'return': 1, 'error': f'Protobuf library path not found in {protobuf_install_path}'}

    return {'return': 0}



import requests
from cmind import utils
import cmind as cm
import os
import json


def preprocess(i):

    os_info = i['os_info']
    env = i['env']
    meta = i['meta']
    automation = i['automation']

    server = env['CM_MLPERF_SUBMISSION_URL']
    benchmark = env['CM_MLPERF_BENCHMARK']
    submitter_id = env['CM_MLPERF_SUBMITTER_ID']
    file_path = env['CM_MLPERF_SUBMISSION_FILE']

    r = get_signed_url(server, benchmark, submitter_id, file_path)
    if r['return'] > 0:
        return r

    signed_url = r['signed_url']
    submission_id = r['submission_id']

    # print(signed_url)
    # print(submission_id)
    r = upload_file_to_signed_url(file_path, signed_url)
    if r['return'] > 0:
        return r

    r = trigger_submission_checker(
        server, submitter_id, benchmark, submission_id)
    if r['return'] > 0:
        return r

    return {'return': 0}


def get_signed_url(server, benchmark, submitter_id, file_path):
    # Define the URL
    url = f"{server}/index/url"

    # Define the headers
    headers = {
        "Content-Type": "application/json"
    }

    # Define the payload
    payload = {
        "submitter_id": submitter_id,
        "benchmark": benchmark,
        "filename": file_path
    }

    try:
        # Make the POST request
        response = requests.post(url, json=payload, headers=headers)

        # Check the response status
        if response.status_code == 200:
            # print("Request successful!")
            # print("Response:", response.json())
            pass
        else:
            # print(f"Request failed with status code {response.status_code}")
            # print("Response:", response.text)
            pass

    except requests.exceptions.RequestException as e:
        return {"return": 1,
                "error": f"An error occurred in connecting to the server: {e}"}

    response_json = response.json()
    # print(response_json)
    # response = json.loads(response_json)
    try:
        signed_url = response_json['signed_url']
        submission_id = response_json['submission_id']
    except Exception as e:
        return {
            "return": 1, "error": f"An error occurred while processing the response: {e}"}

    return {'return': 0, 'signed_url': signed_url,
            'submission_id': submission_id}


def upload_file_to_signed_url(file_path, signed_url):
    """
    Uploads a file to a signed URL using HTTP PUT.

    Parameters:
        file_path (str): The path to the file you want to upload.
        signed_url (str): The pre-signed URL for uploading the file.

    Returns:
        dict: A dictionary with 'status_code' and 'response' keys.
    """
    headers = {
        'Content-Type': 'application/octet-stream',
        'Access-Control-Allow-Headers': '*'
    }

    try:
        # Open the file in binary mode
        with open(file_path, 'rb') as file:
            response = requests.put(
                signed_url,
                data=file,
                headers=headers
            )

        if response.status_code in [200, 201, 204]:
            print("File uploaded successfully!")
            return {
                'return': 0
            }
        else:
            print(
                f"Failed to upload file. Status code: {response.status_code}")
            print("Response:", response.text)

            return {
                'return': response.status_code,
                'error': response.text
            }

    except FileNotFoundError:
        print("Error: File not found.")
        return {
            'return': 400,
            'error': f'''File {file_path} not found'''
        }

    except requests.exceptions.RequestException as e:
        print(f"Request failed: {e}")
        return {
            'return': 500,
            'error': str(e)
        }


def trigger_submission_checker(
        server_url, submitter_id, benchmark, submission_id):
    """
    Sends a POST request with URL-encoded form data.

    Parameters:
        server_url (str): The server endpoint URL (e.g., https://example.com/index).
        submitter_id (str): The ID of the submitter.
        benchmark (str): The benchmark identifier.
        submission_id (str): The submission ID.

    Returns:
        dict: A dictionary containing status code and response content.
    """
    url = f"{server_url}/index"
    headers = {
        "Content-Type": "application/x-www-form-urlencoded"
    }
    payload = {
        "submitter_id": submitter_id,
        "benchmark": benchmark,
        "submission_id": submission_id
    }

    try:
        # Make the POST request with URL-encoded data
        response = requests.post(url, data=payload, headers=headers)

        if response.ok:
            print("Submission Check Request successful!")
            pass
        else:
            print(
                f"Submission Check Request failed with status code: {response.status_code}")
            print("Response:", response.text)

        return {
            "return": 0,
            "response": response.text
        }

    except requests.exceptions.RequestException as e:
        print("An error occurred:", e)
        return {
            "return": 500,
            "error": str(e)
        }


def postprocess(i):
    return {'return': 0}



# Developer(s): Grigori Fursin

import streamlit as st
import os
import cmind

import misc


def main():

    query_params = misc.get_params(st)

    script_path = os.environ.get('CM_GUI_SCRIPT_PATH', '')
    script_alias = os.environ.get('CM_GUI_SCRIPT_ALIAS', '')
    title = os.environ.get('CM_GUI_TITLE', '')

    # Check if script tags are specified from CMD
    script_tags = os.environ.get('CM_GUI_SCRIPT_TAGS', '').strip()

    script_tags_from_url = query_params.get('tags', [''])
    if len(script_tags_from_url) > 0:
        x_script_tags_from_url = script_tags_from_url[0].strip()
        if x_script_tags_from_url != '':
            script_tags = x_script_tags_from_url

    meta = {}

    if script_tags != '':
        # Check type of tags
        if ' ' in script_tags:
            script_tags = script_tags.replace(' ', ',')

        print('Searching CM scripts using tags "{}"'.format(script_tags))

        r = cmind.access({'action': 'find',
                          'automation': 'script,5b4e0237da074764',
                          'tags': script_tags})
        if r['return'] > 0:
            return r

        lst = r['list']

        if len(lst) == 1:
            script = lst[0]
            meta = script.meta
            script_path = script.path
            script_alias = meta['alias']

    # Read meta
    if len(meta) == 0 and script_path != '' and os.path.isdir(script_path):
        fn = os.path.join(script_path, '_cm')
        r = cmind.utils.load_yaml_and_json(fn)
        if r['return'] == 0:
            meta = r['meta']
            script_path = script.path
            script_alias = meta['alias']

    import script

    ii = {'st': st,
          'params': query_params,
          'script_path': script_path,
          'script_alias': script_alias,
          'script_tags': script_tags,
          'script_meta': meta,
          'skip_bottom': False}

    return script.page(ii)


if __name__ == "__main__":
    main()



# Developer(s): Grigori Fursin

import cmind
import os
import misc

import streamlit.components.v1 as components

import streamlit as st

import matplotlib
import matplotlib.pyplot as plt
import matplotlib.colors as mcolors

import numpy as np
import pandas as pd

import mpld3
from mpld3 import plugins
from mpld3 import utils

security = ['os.', 'streamlit.', 'matplotlib.', 'numpy.', 'pandas.', 'mpld3.']


repro_badges = {
    'acm_ctuning_repro_badge_functional': {'img': 'https://cTuning.org/images/artifacts_evaluated_functional_v1_1_small.png'},
    'acm_ctuning_repro_badge_reproduce': {'img': 'https://cTuning.org/images/results_reproduced_v1_1_small.png'},
    'acm_ctuning_repro_badge_support_docker': {'img': 'https://cTuning.org/images/docker_logo2_small.png'},
    'acm_ctuning_repro_badge_cm_interface': {'img': 'https://cTuning.org/images/logo-ck-single-tr4.png'}
}


class OpenBrowserOnClick(mpld3.plugins.PluginBase):

    JAVASCRIPT = """

    mpld3.register_plugin("openbrowseronclick", PointClickableHTMLTooltip);

    PointClickableHTMLTooltip.prototype = Object.create(mpld3.Plugin.prototype);
    PointClickableHTMLTooltip.prototype.constructor = PointClickableHTMLTooltip;
    PointClickableHTMLTooltip.prototype.requiredProps = ["id"];
    PointClickableHTMLTooltip.prototype.defaultProps = {targets:null};

    function PointClickableHTMLTooltip(fig, props){
        mpld3.Plugin.call(this, fig, props);
    };

    PointClickableHTMLTooltip.prototype.draw = function(){
       var obj = mpld3.get_element(this.props.id);
       var targets = this.props.targets;
       obj.elements()
           .on("mousedown", function(d, i){
                              window.open(targets[i]);
                               });
    };

    """

    def __init__(self, points, targets=None):
        self.points = points
        self.targets = targets
        self.dict_ = {"type": "openbrowseronclick",
                      "id": mpld3.utils.get_id(points, None),
                      "targets": targets}


def main():

    params = misc.get_params(st)

    # Set title
    st.title('CM experiment visualization')

    return visualize(st, params)


def visualize(st, query_params, action=''):

    # Query experiment
    result_uid = os.environ.get('CM_GUI_GRAPH_EXPERIMENT_RESULT_UID', '')
    q_result_uid = query_params.get('result_uid', [''])
    if len(q_result_uid) > 0:
        if q_result_uid[0] != '':
            result_uid = q_result_uid[0]

    v_experiment_name = os.environ.get('CM_GUI_GRAPH_EXPERIMENT_NAME', '')
    q_experiment_name = query_params.get('name', [''])
    if len(q_experiment_name) > 0:
        if q_experiment_name[0] != '':
            v_experiment_name = q_experiment_name[0]

    v_experiment_tags = ''
    if v_experiment_name == '':
        v_experiment_tags = os.environ.get('CM_GUI_GRAPH_EXPERIMENT_TAGS', '')
        q_experiment_tags = query_params.get('tags', [''])
        if len(q_experiment_tags) > 0:
            if q_experiment_tags[0] != '':
                v_experiment_tags = q_experiment_tags[0]
        v_experiment_tags = v_experiment_tags.replace(',', ' ')

        # Check default
#        if v_experiment_tags == '' and v_experiment_name == '':
#            v_experiment_tags = 'mlperf-inference v4.0'

        v_experiment_tags = st.text_input(
            'Select CM experiment tags separated by space:',
            value=v_experiment_tags,
            key='v_experiment_tags').strip()
        v_experiment_tags = v_experiment_tags.replace(',', ' ')

    # Get all experiment names
    ii = {'action': 'find',
          'automation': 'experiment,a0a2d123ef064bcb'}

    # If name is given, do not use tags
    if v_experiment_name != '':
        ii['artifact'] = v_experiment_name
    elif v_experiment_tags != '':
        ii['tags'] = v_experiment_tags.replace(' ', ',')

    r = cmind.access(ii)
    if r['return'] > 0:
        return r

    lst_all = r['list']

    experiments = ['']

    selection = 0
    index = 1
    for l in sorted(lst_all, key=lambda x: (
        ','.join(x.meta.get('tags', [])),
        x.meta.get('alias', ''),
        x.meta['uid']
    )):

        meta = l.meta

        if v_experiment_name != '' and (
                v_experiment_name == meta['alias'] or v_experiment_name == meta['uid']):
            selection = index

        name = ' '.join(meta.get('tags', []))
        if name == '':
            name = meta.get('alias', '')
        if name == '':
            name = meta['uid']

        experiments.append(name)

        index += 1

    if len(lst_all) == 1:
        selection = 1

    # Show experiment artifacts
    experiment = st.selectbox('Select experiment from {} found:'.format(len(experiments) - 1),
                              range(len(experiments)),
                              format_func=lambda x: experiments[x],
                              index=selection,
                              key='experiment')

    lst = [lst_all[experiment - 1]] if experiment > 0 else lst_all

    if len(lst) > 8:
        st.markdown('Too many experiments - continue pruning ...')
        return {'return': 0}

    # Check experiments
    results = []
    results_with_password = []
    passwords = []
    results_meta = {}

    for experiment in lst:
        path = experiment.path

        for d in os.listdir(path):
            path2 = os.path.join(path, d)
            if os.path.isdir(path2):
                path_to_result = os.path.join(path, d, 'cm-result.json')

                if os.path.isfile(path_to_result):
                    emeta = experiment.meta

                    desc = {'path': path_to_result,
                            'experiment_dir': d,
                            'experiment_uid': emeta['uid'],
                            'experiment_alias': emeta['alias'],
                            'experiment_tags': ','.join(emeta.get('tags', []))}

                    add = True
                    if result_uid != '':
                        add = False
                        r = cmind.utils.load_json(path_to_result)
                        if r['return'] == 0:
                            meta = r['meta']

                            results_meta[path_to_result] = meta

                            for m in meta:
                                if m.get('uid', '') == result_uid:
                                    add = True
                                    break

                    if add:
                        pwd = experiment.meta.get('password_hash', '')
                        if pwd == '':
                            results.append(desc)
                        else:
                            desc['password_hash'] = pwd

                            if pwd not in passwords:
                                passwords.append(pwd)

                            results_with_password.append(desc)

    # Check if password
    if len(passwords) > 0:
        password = st.text_input(
            'Some results are protected by password. Enter password to unlock them:',
            value='',
            key='v_experiment_pwd').strip()

        if password != '':
            import bcrypt
            # salt = bcrypt.gensalt()
            # TBD: temporal hack to demo password protection for experiments
            # salt = bcrypt.gensalt()
            password_salt = b'$2b$12$ionIRWe5Ft7jkn4y/7C6/e'
            password_hash2 = bcrypt.hashpw(
                password.encode('utf-8'),
                password_salt).decode('utf-8')

            for result in results_with_password:
                if result['password_hash'] == password_hash2:
                    results.append(result)

    # How to visualize selection
    if len(results) == 0:
        st.markdown('No results found!')
        return {'return': 0}

    if st.session_state.get('tmp_cm_results', '') == '':
        st.session_state['tmp_cm_results'] = len(results)
    elif int(st.session_state['tmp_cm_results']) != len(results):
        st.session_state['tmp_cm_results'] = len(results)
        st.session_state['how'] = 0

    how = ''

    if result_uid == '':
        v_max_results = os.environ.get(
            'CM_GUI_GRAPH_EXPERIMENT_MAX_RESULTS', '')

        if v_max_results != '' and len(results) > int(v_max_results):
            st.markdown('Too many results - continue pruning ...')
            return {'return': 0}

        v_how = os.environ.get('CM_GUI_GRAPH_EXPERIMENT_HOW', '')
        q_how = query_params.get('type', [''])
        if len(q_how) > 0:
            if q_how[0] != '':
                v_how = q_how[0]

        how_selection = ['', '2d-static', '2d', 'bar']
        how_selection_desc = [
            '',
            'Scatter plot (static)',
            'Scatter plot (interactive, slow - to be improved)',
            'Bar plot (static)']

        how_index = 0
        if v_how != '' and v_how in how_selection:
            how_index = how_selection.index(v_how)

        how2 = st.selectbox('Select how to visualize {} CM experiment set(s):'.format(len(results)),
                            range(len(how_selection_desc)),
                            format_func=lambda x: how_selection_desc[x],
                            index=how_index,
                            key='how')

        if how2 == '' or how2 == 0:
            return {'return': 0}

        how = how_selection[how2]

    how = how.strip()

    # Continue visualizing
    all_values = []
    keys = []
    all_data = []

    derived_metrics_value = query_params.get(
        'derived_metrics', [''])[0].strip()
    derived_metrics_value = st.text_input("Optional: add derived metrics in Python. Example: result['Accuracy2'] = result['Accuracy']*2",
                                          value=derived_metrics_value).strip()

    for x in security:
        if x in derived_metrics_value:
            derived_metrics_value = ''
            break

    error_shown2 = False
    for desc in results:
        path_to_result = desc['path']

        if path_to_result in results_meta:
            result_meta = results_meta[path_to_result]
        else:
            r = cmind.utils.load_json_or_yaml(path_to_result)
            if r['return'] > 0:
                return r

            result_meta = r['meta']

        for result in result_meta:
            # Add extra info
            for k in ['experiment_dir', 'experiment_alias',
                      'experiment_uid', 'experiment_tags']:
                if k in desc:
                    result[k] = desc[k]

            if derived_metrics_value != '':
                try:
                    exec(derived_metrics_value)
                except Exception as e:
                    if not error_shown2:
                        st.markdown(
                            '*Syntax error in derived metrics: {}*'.format(e))
                        error_shown2 = True

            all_values.append(result)

            for k in result.keys():
                if k not in keys:
                    keys.append(k)

    first_keys = [
        'Organization',
        'Model',
        'Scenario',
        'SystemName',
        'notes',
        'framework',
        'Result',
        'Result_Units',
        'Accuracy']
    sorted_keys = [k for k in first_keys if k in keys] + \
        [k for k in sorted(keys, key=lambda s: s.lower())
         if k not in first_keys]

    filter_value = query_params.get('filter', [''])[0].strip()
    if result_uid == '':  # and filter_value!='':
        filter_value = st.text_input(
            "Optional: add result filter in Python. Examples: result['Accuracy']>75 or 'llama2' in result['Model']",
            value=filter_value).strip()

        st.markdown('---')

    for x in security:
        if x in filter_value:
            filter_value = ''
            break

    # all_values is a list of dictionaries with all keys
    error_shown = False
    for result in all_values:

        if filter_value != '':
            try:
                if not eval(filter_value):
                    continue
            except Exception as e:
                if not error_shown:
                    st.markdown('*Syntax error in filter: {}*'.format(e))
                    error_shown = True

        # Check if 1 result UID is selected
        if result_uid != '' and result.get('uid', '') != result_uid:
            continue

        data = []
        for k in sorted_keys:
            data.append(result.get(k))

        all_data.append(data)

        if result_uid != '':
            break

    ###################################################
    if len(all_data) == 0:
        st.markdown('No results found for your selection.')
        return {'return': 0, 'end_html': end_html}

    ###################################################
    # If experiment found and 1 UID, print a table
    if result_uid != '':
        st.markdown('---')
        st.markdown('# Result summary')

        data = all_data[0]

        result = {}

        j = 0
        for k in sorted_keys:
            result[k] = data[j]
            j += 1

        # Check badges
        x = ''

        for k in repro_badges:
            if result.get(k, False):
                img = repro_badges[k]['img']

                x += '<a href="http://cTuning.org/ae" target="_blank"><img src="{}" height="64"></a>\n'.format(
                    img)

        if x != '':
            st.write(
                '<center>\n' + x + '\n</center>\n',
                unsafe_allow_html=True)

        x = ''
        for k in sorted_keys:
            x += '* **{}**: {}\n'.format(k, str(result[k]))

        st.markdown(x)

        # Check associated reports
        r = cmind.access({'action': 'find',
                          'automation': 'report,6462ecdba2054467',
                         'tags': 'result-{}'.format(result_uid)})
        if r['return'] > 0:
            return r

        lst = r['list']

        for l in lst:
            report_path = l.path

            f1 = os.path.join(report_path, 'README.md')
            if os.path.isfile(f1):
                report_meta = l.meta

                report_alias = report_meta['alias']
                report_title = report_meta.get('title', '')

                report_name = report_title if report_title != '' else report_alias

                r = cmind.utils.load_txt(f1)
                if r['return'] > 0:
                    return r

                s = r['string']

                st.markdown('---')
                st.markdown('### ' + report_name)

                st.markdown(s)

        # Create self link
        st.markdown("""---""")

        experiment_alias_or_uid = result['experiment_uid']

        end_html = '''
         <center>
          <small><a href="{}&result_uid={}"><i>Self link</i></a></small>
         </center>
         '''.format(misc.make_url(experiment_alias_or_uid, action=action, md=False), result_uid)

        st.write(end_html, unsafe_allow_html=True)

        return {'return': 0}

    ###################################################
    # Select 2D keys
    axis_key_x = ''
    axis_key_y = ''
    axis_key_c = ''

    if len(keys) > 0:
        keys = [''] + sorted_keys

        axis_key_x = os.environ.get('CM_GUI_GRAPH_EXPERIMENT_AXIS_KEY_X', '')
        q_axis_key_x = query_params.get('x', [''])
        if len(q_axis_key_x) > 0:
            if q_axis_key_x[0] != '':
                axis_key_x = q_axis_key_x[0]
        i_axis_key_x = 0
        if axis_key_x != '' and axis_key_x in keys:
            i_axis_key_x = keys.index(axis_key_x)
        if axis_key_x == '' and 'Result' in keys:
            i_axis_key_x = keys.index('Result')
        axis_key_x = st.selectbox(
            'Select X key', keys, index=i_axis_key_x, key='x')

        axis_key_y = os.environ.get('CM_GUI_GRAPH_EXPERIMENT_AXIS_KEY_Y', '')
        q_axis_key_y = query_params.get('y', [''])
        if len(q_axis_key_y) > 0:
            if q_axis_key_y[0] != '':
                axis_key_y = q_axis_key_y[0]
        i_axis_key_y = 0
        if axis_key_y != '' and axis_key_y in keys:
            i_axis_key_y = keys.index(axis_key_y)
        if axis_key_y == '' and 'Accuracy' in keys:
            i_axis_key_y = keys.index('Accuracy')
        axis_key_y = st.selectbox(
            'Select Y key', keys, index=i_axis_key_y, key='y')

        axis_key_c = os.environ.get('CM_GUI_GRAPH_EXPERIMENT_AXIS_KEY_C', '')
        q_axis_key_c = query_params.get('c', [''])
        if len(q_axis_key_c) > 0:
            if q_axis_key_c[0] != '':
                axis_key_c = q_axis_key_c[0]
        i_axis_key_c = 0
        if axis_key_c != '' and axis_key_c in keys:
            i_axis_key_c = keys.index(axis_key_c)
        if axis_key_c == '' and 'version' in keys:
            i_axis_key_c = keys.index('version')
        axis_key_c = st.selectbox(
            'Select Color key',
            keys,
            index=i_axis_key_c,
            key='c')

        axis_key_s = os.environ.get('CM_GUI_GRAPH_EXPERIMENT_AXIS_KEY_S', '')
        q_axis_key_s = query_params.get('s', [''])
        if len(q_axis_key_s) > 0:
            axis_key_s = q_axis_key_s[0]
        i_axis_key_s = 0
        if axis_key_s != '' and axis_key_s in keys:
            i_axis_key_s = keys.index(axis_key_s)
        axis_key_s = st.selectbox(
            'Select Style key',
            keys,
            index=i_axis_key_s,
            key='s')

    # Select values
    values = []

    if axis_key_x != '' and axis_key_y != '':
        for v in all_values:
            x = v.get(axis_key_x, None)
            y = v.get(axis_key_y, None)

            if x is not None and y is not None:
                values.append(v)

    if len(values) > 0:

        # fig, ax = plt.subplots(figsize=(12,6))
        fig, ax = plt.subplots()  # figsize=(6,4))

        ax.set_xlabel(axis_key_x)
        ax.set_ylabel(axis_key_y)

        title = os.environ.get('CM_GUI_GRAPH_EXPERIMENT_TITLE', '')
        q_title = query_params.get('title', [''])
        if len(q_title) > 0:
            if q_title[0] != '':
                title = q_title[0]
        ax.set_title(title, size=16)

        if how == 'bar':
            ax.set_title('Under development ...', size=16)
            ax.yaxis.grid(linestyle='dotted')
        else:
            ax.grid(linestyle='dotted')
        # https://matplotlib.org/stable/api/markers_api.html

        unique_color_values = {}
#        unique_colors = list(mcolors.CSS4_COLORS.keys())
        unique_colors = list(mcolors.TABLEAU_COLORS.keys())
        i_unique_color_values = 0

        unique_style_values = {}
#        unique_styles = ['o','v','^','<','>','1','2','3','4','8','s','p','P','*','+','D']
        unique_styles = ['circle', 'square', 'diamond', 'cross', 'x', 'triangle', 'pentagon', 'hexagram',
                         'star', 'hourglass', 'bowtie', 'asterisk', 'hash']
        i_unique_style_values = 0

        # If Bar, use Style to separate results
        unique_x_values = []
        unique_s_values = []

        experiment_uids = []

        # Filter values
        values2 = []

        for result in values:
            if filter_value != '':
                try:
                    if not eval(filter_value):
                        continue
                except Exception as e:
                    if not error_shown:
                        st.markdown('*Syntax error in filter: {}*'.format(e))
                        error_shown = True

            values2.append(result)

            if how == 'bar':
                x = result.get(axis_key_x, None)
                if x is not None and x != '' and x not in unique_x_values:
                    unique_x_values.append(x)

                s = result.get(axis_key_s, None)
                if s is not None and s != '' and s not in unique_s_values:
                    unique_s_values.append(s)

        #######################################################################
        # Continue visualizing
        if how == '2d-static' or how == 'bar':

            xx = []
            yy = []
            cc = []
            ss = []
            io = []

            t = 0
            for result in values2:
                v = result

                t += 1

                x = v.get(axis_key_x, None)
                y = v.get(axis_key_y, None)

                xx.append(x)
                yy.append(y)

                color = 'blue'
                if axis_key_c != '':
                    c = v.get(axis_key_c, None)
                    if c is not None:
                        if c in unique_color_values:
                            color = unique_color_values[c]
                        else:
                            color = unique_colors[i_unique_color_values]
                            unique_color_values[c] = color
                            if i_unique_color_values < (
                                    len(unique_colors) - 1):
                                i_unique_color_values += 1

                cc.append(color)

                style = 'o'
                if axis_key_s != '':
                    s = v.get(axis_key_s, None)
                    if s is not None:
                        if s in unique_style_values:
                            style = unique_style_values[s]
                        else:
                            style = unique_styles[i_unique_style_values]
                            unique_style_values[s] = style
                            if i_unique_style_values < (
                                    len(unique_styles) - 1):
                                i_unique_style_values += 1

                ss.append(style)

                info = ''
                for key in sorted(v.keys(), key=lambda x: x.lower()):
                    value = v[key]
                    info += str(key) + ': ' + str(value) + '<br>\n'

                io.append(info)

            import plotly.express as px

            dd = {
                axis_key_x: xx,
                axis_key_y: yy,
                axis_key_c: cc,
                axis_key_s: ss,
                'info': io}

            # https://docs.streamlit.io/library/api-reference/charts/st.bar_chart
            # https://docs.streamlit.io/library/api-reference/charts/st.plotly_chart
            # https://plotly.com/python/line-and-scatter/

            df = pd.DataFrame(dd)

            if how == 'bar':
                st.bar_chart(df, x=axis_key_x, y=axis_key_y)
            else:
                fig = px.scatter(
                    df,
                    x=axis_key_x,
                    y=axis_key_y,
                    color=axis_key_c,
                    symbol=axis_key_s,
                    hover_name='info',
                    height=1000)

                st.plotly_chart(
                    fig,
                    theme="streamlit",
                    use_container_width=True)

        elif how == '2d':
            ###################################################################
            # 2D interactive graph - very slow - need to be updated
            width = 1

            t = 0
            for result in values2:
                v = result

                t += 1

                x = v.get(axis_key_x, None)
                y = v.get(axis_key_y, None)

                url = v.get('url', '')
                if url == '':
                    url = v.get('git_url', '')

                color = 'blue'
                if axis_key_c != '':
                    c = v.get(axis_key_c, None)
                    if c is not None:
                        if c in unique_color_values:
                            color = unique_color_values[c]
                        else:
                            color = unique_colors[i_unique_color_values]
                            unique_color_values[c] = color
                            if i_unique_color_values < (
                                    len(unique_colors) - 1):
                                i_unique_color_values += 1

                style = 'o'
                if axis_key_s != '':
                    s = v.get(axis_key_s, None)
                    if s is not None:
                        if s in unique_style_values:
                            style = unique_style_values[s]
                        else:
                            style = unique_styles[i_unique_style_values]
                            unique_style_values[s] = style
                            if i_unique_style_values < (
                                    len(unique_styles) - 1):
                                i_unique_style_values += 1

                graph = ax.scatter(x, y, color=color, marker=style)

                info = ''
                for key in sorted(v.keys(), key=lambda x: x.lower()):
                    value = v[key]
                    info += '<b>' + str(key) + ': </b>' + str(value) + '<br>\n'

                info2 = '<div style="padding:10px;background-color:#FFFFE0;"><small>' + \
                    info + '</small></div>'

                label = [info2]
                plugins.connect(fig, plugins.PointHTMLTooltip(graph, label))

                experiment_uid = v.get('experiment_uid', '')
                if experiment_uid != '' and experiment_uid not in experiment_uids:
                    experiment_uids.append(experiment_uid)

                uid = v.get('uid', '')
                if uid != '':
                    xaction = 'action={}&'.format(
                        action) if action != '' else ''
                    url = '?{}name={}&result_uid={}'.format(
                        xaction, experiment_uid, uid)

                if url != '':
                    targets = [url]
                    plugins.connect(
                        fig, OpenBrowserOnClick(
                            graph, targets=targets))

            # Render graph
            fig_html = mpld3.fig_to_html(fig)
            components.html(fig_html, width=1100, height=500)

            # fig_html = '<div
            # style="padding:10px;background-color:#F0F0F0;">'+fig_html+'</div>'

            # components.html(fig_html, width=1000, height=800)
            # st.markdown('---')

        #######################################################################
        # Show all data
        df = pd.DataFrame(
            all_data,
            columns=(k for k in sorted_keys if k != '')
        )

        st.markdown('---')
        st.dataframe(df)

        # Check if can create self link
        if len(experiment_uids) == 1:
            st.markdown("""---""")

            xtype = '&type={}'.format(how) if how != '' else ''

            end_html = '''
             <center>
              <small><a href="{}{}"><i>Self link</i></a></small>
             </center>
             '''.format(misc.make_url(experiment_uids[0], action=action, md=False), xtype)

            st.write(end_html, unsafe_allow_html=True)

    return {'return': 0}


if __name__ == "__main__":
    r = main()

    if r['return'] > 0:

        st.markdown("""---""")
        st.markdown('**Error detected by CM:** {}'.format(r['error']))



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']
    env['OMP_PROC_BIND'] = 'true'

    meta = i['meta']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    return {'return': 0}


def postprocess(i):

    env = i['env']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import cmind as cm
import os
from os.path import exists
import shutil


def preprocess(i):

    os_info = i['os_info']
    env = i['env']
    submission_dir = env.get("CM_MLPERF_INFERENCE_SUBMISSION_DIR", "")

    if submission_dir == "":
        print("Please set --env.CM_MLPERF_INFERENCE_SUBMISSION_DIR")
        return {'return': 1,
                'error': 'CM_MLPERF_INFERENCE_SUBMISSION_DIR is not specified'}

    if not os.path.exists(submission_dir):
        print("Please set --env.CM_MLPERF_INFERENCE_SUBMISSION_DIR to a valid submission directory")
        return {'return': 1,
                'error': 'CM_MLPERF_INFERENCE_SUBMISSION_DIR is not existing'}

    submission_dir = submission_dir.rstrip(os.path.sep)
    submitter = env.get("CM_MLPERF_SUBMITTER", "MLCommons")
    submission_processed = f"{submission_dir}_processed"

    if os.path.exists(submission_processed):
        print(f"Cleaning {submission_processed}")
        shutil.rmtree(submission_processed)

    version = env.get('CM_MLPERF_SUBMISSION_CHECKER_VERSION', '')
    x_version = ' --version ' + version + ' ' if version != '' else ''

    CMD = env['CM_PYTHON_BIN'] + " '" + os.path.join(env['CM_MLPERF_INFERENCE_SOURCE'], "tools", "submission",
                                                     "preprocess_submission.py") + "' --input '" + submission_dir + "' --submitter '" + submitter + "' --output '" + submission_processed + "'" + x_version
    env['CM_RUN_CMD'] = CMD

    return {'return': 0}


def postprocess(i):

    env = i['env']
    submission_dir = env["CM_MLPERF_INFERENCE_SUBMISSION_DIR"]
    import datetime
    submission_backup = submission_dir + "_backup_" + \
        '{date:%Y-%m-%d_%H:%M:%S}'.format(date=datetime.datetime.now())

    submission_processed = submission_dir + "_processed"
    shutil.copytree(submission_dir, submission_backup)
    shutil.rmtree(submission_dir)
    os.rename(submission_processed, submission_dir)

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import cmind as cm


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    cmd = env.get('CM_GH_ACTIONS_RUNNER_COMMAND', '')
    if cmd == "config":
        run_cmd = f"cd {env['CM_GH_ACTIONS_RUNNER_CODE_PATH']} && ./config.sh --url {env['CM_GH_ACTIONS_RUNNER_URL']} --token {env['CM_GH_ACTIONS_RUNNER_TOKEN']}"
    elif cmd == "remove":
        run_cmd = f"cd {env['CM_GH_ACTIONS_RUNNER_CODE_PATH']} && ./config.sh remove --token {env['CM_GH_ACTIONS_RUNNER_TOKEN']}"
    elif cmd == "install":
        run_cmd = f"cd {env['CM_GH_ACTIONS_RUNNER_CODE_PATH']} && sudo ./svc.sh install"
    elif cmd == "uninstall":
        run_cmd = f"cd {env['CM_GH_ACTIONS_RUNNER_CODE_PATH']} && sudo ./svc.sh uninstall"
        cache_rm_tags = "gh,runner,_install"
        r = cm.access({'action': 'rm', 'automation': 'cache',
                      'tags': cache_rm_tags, 'f': True})
        print(r)
        if r['return'] != 0 and r['return'] != 16:  # ignore missing ones
            return r
    elif cmd == "start":
        run_cmd = f"cd {env['CM_GH_ACTIONS_RUNNER_CODE_PATH']} && sudo ./svc.sh start"

    env['CM_RUN_CMD'] = run_cmd

    return {'return': 0}


def postprocess(i):

    env = i['env']

    return {'return': 0}



import abc
import typing

import numpy as np

ModelInput = typing.Dict[str, np.array]


class Model(abc.ABC):
    @abc.abstractmethod
    def predict(self, input: ModelInput) -> typing.Any:
        pass


class ModelFactory(abc.ABC):
    @abc.abstractmethod
    def create(self) -> Model:
        pass


class ModelInputSampler(abc.ABC):
    @abc.abstractmethod
    def sample(self, id: int) -> ModelInput:
        pass



from huggingface_hub import hf_hub_download
import os

model_stub = os.environ.get('CM_MODEL_ZOO_STUB', '')
model_task = os.environ.get('CM_MODEL_TASK', '')

revision = os.environ.get('CM_HF_REVISION', '')

if model_task == "prune":
    print("Downloading model: " + model_stub)

    for filename in ["pytorch_model.bin", "config.json"]:

        downloaded_model_path = hf_hub_download(repo_id=model_stub,
                                                filename=filename,
                                                cache_dir=os.getcwd())

    with open('tmp-run-env.out', 'w') as f:
        f.write(f"CM_ML_MODEL_FILE_WITH_PATH={os.path.join(os.getcwd(),'')}")

else:
    subfolder = os.environ.get('CM_HF_SUBFOLDER', '')
    full_subfolder = os.environ.get('CM_HF_FULL_SUBFOLDER', '')

    model_filename = os.environ.get('CM_MODEL_ZOO_FILENAME', '')
    if model_filename == '':
        model_filename = 'model.onnx'

    model_filenames = model_filename.split(
        ',') if ',' in model_filename else [model_filename]

    base_model_filepath = None

    files = []
    if full_subfolder != '':

        from huggingface_hub import HfFileSystem
        fs = HfFileSystem()

        # List all files in a directory
        path = model_stub + '/' + full_subfolder

        print('')
        print('Listing files in {} ...'.format(path))

        def list_hf_files(path):
            all_files = []

            xrevision = None if revision == '' else revision
            files = fs.ls(path, revision=xrevision)  # , detail=False)

            for f in files:
                fname = f['name']
                fdir = f['type'] == 'directory'

                if fdir:
                    all_files += list_hf_files(fname)
                else:
                    all_files.append(fname)

            return all_files

        files = list_hf_files(path)

        print('')
        print('Found {} files'.format(len(files)))

        for f in files:

            remove = len(model_stub) + 1

            if revision != '':
                remove += len(revision) + 1

            ff = f[remove:]

            if ff not in model_filenames:
                model_filenames.append(ff)

    print('')
    for model_filename in model_filenames:

        print("Downloading file {} / {} ...".format(model_stub, model_filename))

        extra_dir = os.path.dirname(model_filename)

        if extra_dir != '' and not os.path.exists(extra_dir):
            os.makedirs(extra_dir)

        xrevision = None if revision == '' else revision
        xsubfolder = None if subfolder == '' else subfolder

        downloaded_path = hf_hub_download(repo_id=model_stub,
                                          subfolder=xsubfolder,
                                          filename=model_filename,
                                          revision=xrevision,
                                          cache_dir=os.getcwd())
        print(downloaded_path)
        if not base_model_filepath:
            base_model_filepath = downloaded_path

    print('')

    with open('tmp-run-env.out', 'w') as f:
        f.write(f"CM_ML_MODEL_FILE_WITH_PATH={base_model_filepath}")



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import cmind as cm


def preprocess(i):

    os_info = i['os_info']
    env = i['env']
    meta = i['meta']
    automation = i['automation']
    run_script_input = i['run_script_input']
    pip_version = env.get('CM_PIP_VERSION', '').strip().split('.')

    package_name = env.get('CM_GENERIC_PYTHON_PACKAGE_NAME', '').strip()
    if package_name == '':
        return automation._available_variations({'meta': meta})

    if package_name == "onnxruntime_gpu":
        # https://onnxruntime.ai/docs/execution-providers/CUDA-ExecutionProvider.html#requirements
        # 20240214: ONNXRuntime 1.17.0 now support CUDA 12 so we remove next check
        # TBD: if we have explicit version for ONNX < 17.0.0 and CUDA is >= 12,
        # we should add a check to fail ...
        cuda_version = env.get('CM_CUDA_VERSION', '').strip()
#        if cuda_version!='':
#            cuda_version_split = cuda_version.split('.')
#            if int(cuda_version_split[0]) >= 12:
#                # env['CM_INSTALL_ONNXRUNTIME_GPU_FROM_SRC'] = "yes"
# return {'return': 1, 'error':'at this moment, PIP package
# "onnxruntime_gpu" needs CUDA < 12'}

    extra = env.get('CM_GENERIC_PYTHON_PIP_EXTRA', '')
    if (pip_version and len(pip_version) > 1 and int(pip_version[0]) >= 23) and (
            '--break-system-packages' not in extra):
        extra += '  --break-system-packages '
        env['CM_PYTHON_PIP_COMMON_EXTRA'] = " --break-system-packages"

    if env.get('CM_GENERIC_PYTHON_PACKAGE_INSTALL_DEPS', '') == "no":
        env['CM_PYTHON_PIP_COMMON_EXTRA'] = " --no-deps"

    if env.get('CM_PIP_INSTALL_NEEDS_USER', '') == "yes":
        env['CM_PYTHON_PIP_COMMON_EXTRA'] = " --user"

    if env.get('CM_GENERIC_PYTHON_PIP_UNINSTALL_DEPS', '') != '':
        r = automation.run_native_script(
            {'run_script_input': run_script_input, 'env': env, 'script_name': 'uninstall_deps'})
        if r['return'] > 0:
            return r

    prepare_env_key = env.get('CM_GENERIC_PYTHON_PACKAGE_NAME', '')
    for x in ["-", "[", "]"]:
        prepare_env_key = prepare_env_key.replace(x, "_")

    env['CM_TMP_PYTHON_PACKAGE_NAME_ENV'] = prepare_env_key.upper()

    recursion_spaces = i['recursion_spaces']

    r = automation.detect_version_using_script({
        'env': env,
        'run_script_input': i['run_script_input'],
        'recursion_spaces': recursion_spaces})

    force_install = (
        env.get(
            'CM_TMP_PYTHON_PACKAGE_FORCE_INSTALL',
            '') in [
            'yes',
            'true',
            'True',
            True])

    if r['return'] > 0 or force_install:
        if r['return'] == 16 or force_install:
            # Clean detected version env if exists otherwise takes detected version
            # for example, when we reinstall generic python lib package
            env_version_key = 'CM_' + \
                env['CM_TMP_PYTHON_PACKAGE_NAME_ENV'].upper() + '_VERSION'
            if env.get(env_version_key, '') != '':
                del (env[env_version_key])

            # Check if upgrade
            if force_install:
                extra += ' --upgrade --no-deps --force-reinstall'

            # Check index URL
            index_url = env.get('CM_GENERIC_PYTHON_PIP_INDEX_URL', '').strip()
            if index_url != '':
                # Check special cases
                if '${CM_TORCH_CUDA}' in index_url:
                    index_url = index_url.replace(
                        '${CM_TORCH_CUDA}', env.get('CM_TORCH_CUDA'))

                extra += ' --index-url ' + index_url

            # Check extra index URL
            extra_index_url = env.get(
                'CM_GENERIC_PYTHON_PIP_EXTRA_INDEX_URL', '').strip()

            if extra_index_url != '':
                # Check special cases
                if '${CM_TORCH_CUDA}' in extra_index_url:
                    extra_index_url = extra_index_url.replace(
                        '${CM_TORCH_CUDA}', env.get('CM_TORCH_CUDA'))

                extra += ' --extra-index-url ' + extra_index_url

            # check find-links
            find_links_url = env.get(
                'CM_GENERIC_PYTHON_PIP_EXTRA_FIND_LINKS_URL', '').strip()

            if find_links_url != '':
                extra += ' -f ' + find_links_url

            # Check update
            if env.get('CM_GENERIC_PYTHON_PIP_UPDATE', '') in [
                    True, 'true', 'yes', 'on']:
                extra += ' -U'

            print('')
            print(recursion_spaces + '      Extra PIP CMD: ' + extra)
            print('')

            env['CM_GENERIC_PYTHON_PIP_EXTRA'] = extra

            r = automation.run_native_script(
                {'run_script_input': run_script_input, 'env': env, 'script_name': 'install'})

            if r['return'] > 0:
                return r

    return {'return': 0}


def detect_version(i):

    env = i['env']

    if env.get('CM_TMP_PYTHON_PACKAGE_NAME_ENV', '') != '':
        env_version_key = 'CM_' + \
            env['CM_TMP_PYTHON_PACKAGE_NAME_ENV'].upper() + '_VERSION'
    else:
        env_version_key = 'CM_CACHE_TMP_VERSION'

    r = i['automation'].parse_version({'match_text': r'\s*([\d.a-z\-]+)',
                                       'group_number': 1,
                                       'env_key': env_version_key,
                                       'which_env': i['env']})
    if r['return'] > 0:
        return r

    version = r['version']
    current_detected_version = version

    if env.get('CM_TMP_SILENT', '') != 'yes':
        print(
            i['recursion_spaces'] +
            '      Detected version: {}'.format(version))

    return {'return': 0, 'version': version}


def postprocess(i):

    env = i['env']

    env_version_key = 'CM_' + \
        env['CM_TMP_PYTHON_PACKAGE_NAME_ENV'].upper() + '_VERSION'

    if env.get(env_version_key, '') != '':
        version = env[env_version_key]
    else:
        r = detect_version(i)
        if r['return'] > 0:
            return r

        version = r['version']

    env['CM_PYTHONLIB_' + env['CM_TMP_PYTHON_PACKAGE_NAME_ENV'] +
        '_CACHE_TAGS'] = 'version-' + version

    import pkgutil
    package_name = env.get('CM_GENERIC_PYTHON_PACKAGE_NAME', '').strip()
    package = pkgutil.get_loader(package_name)
    if package:
        installed_file_path = package.get_filename()
        env['CM_GET_DEPENDENT_CACHED_PATH'] = installed_file_path

    pip_version = env.get('CM_PIP_VERSION', '').strip().split('.')
    if pip_version and len(pip_version) > 1 and int(pip_version[0]) >= 23:
        env['CM_PYTHON_PIP_COMMON_EXTRA'] = " --break-system-packages"

    if version.count('.') > 1:
        env[f"{env_version_key}_MAJOR_MINOR"] = ".".join(
            version.split(".")[:2])

    return {'return': 0, 'version': version}



# Developer(s): Grigori Fursin

import cmind
import os
import misc

import streamlit.components.v1 as components

import streamlit as st

announcement = 'Under development - please get in touch via [Discord](https://discord.gg/JjWNWXKxwT) for more details ...'

initialized = False
external_module_path = ''
external_module_meta = {}


def main():
    params = misc.get_params(st)

    # Set title
    st.title('How to run benchmarks')

    st.markdown(announcement)

    return page(st, params)


def page(st, params, action=''):

    global initialized, external_module_path, external_module_meta

    end_html = ''

    st.markdown('----')
    st.markdown(announcement)

    return {'return': 0, 'end_html': end_html}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil
import glob


def preprocess(i):

    env = i['env']

    if 'CM_DATASET_PREPROCESSED_PATH' not in env:
        env['CM_DATASET_PREPROCESSED_PATH'] = os.getcwd()

    if env.get('CM_DATASET_REFERENCE_PREPROCESSOR', "0") == "1":
        print("Using MLCommons Inference source from '" +
              env['CM_MLPERF_INFERENCE_SOURCE'] + "'")

    if env.get('CM_ML_MODEL_NAME', '') == 'retinanet':
        if env.get('CM_DATASET_QUANTIZE', '') == '1':
            if env.get('CM_QAIC_MODEL_RETINANET_IMAGE_SCALE', '') != '':
                env['CM_DATASET_QUANT_SCALE'] = env['CM_QAIC_MODEL_RETINANET_IMAGE_SCALE']
            if env.get('CM_QAIC_MODEL_RETINANET_IMAGE_OFFSET', '') != '':
                env['CM_DATASET_QUANT_OFFSET'] = env['CM_QAIC_MODEL_RETINANET_IMAGE_OFFSET']

    return {'return': 0}


def postprocess(i):

    env = i['env']

    if env["CM_DATASET_TYPE"] == "validation":
        env['CM_DATASET_ANNOTATIONS_DIR_PATH'] = os.path.join(
            env['CM_DATASET_PREPROCESSED_PATH'], "annotations")
        env['CM_DATASET_ANNOTATIONS_FILE_PATH'] = os.path.join(
            env['CM_DATASET_ANNOTATIONS_DIR_PATH'], "openimages-mlperf.json")

    # finalize path
    preprocessed_path = env['CM_DATASET_PREPROCESSED_PATH']
    preprocessed_images_list = []
    preprocessed_imagenames_list = []

    match_text = "/*." + env.get("CM_DATASET_PREPROCESSED_EXTENSION", "*")
    for filename in sorted(glob.glob(preprocessed_path + match_text)):
        preprocessed_images_list.append(filename)
        preprocessed_imagenames_list.append(os.path.basename(filename))
    with open("preprocessed_files.txt", "w") as f:
        f.write("\n".join(preprocessed_images_list))
    with open("preprocessed_filenames.txt", "w") as f:
        f.write("\n".join(preprocessed_imagenames_list))

    env['CM_DATASET_PREPROCESSED_IMAGES_LIST'] = os.path.join(
        os.getcwd(), "preprocessed_files.txt")
    env['CM_DATASET_PREPROCESSED_IMAGENAMES_LIST'] = os.path.join(
        os.getcwd(), "preprocessed_filenames.txt")

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    # Pre-set by CM
    os_info = i['os_info']
    env = i['env']
    recursion_spaces = i['recursion_spaces']
    automation = i['automation']
    run_script_input = i['run_script_input']

    # Check if a given tool is already installed
    file_name_core = 'aria2c'
    file_name = file_name_core + \
        '.exe' if os_info['platform'] == 'windows' else file_name_core

    force_install = env.get('CM_FORCE_INSTALL', False) == True

    if not force_install:
        r = i['automation'].find_artifact({'file_name': file_name,
                                           'env': env,
                                           'os_info': os_info,
                                           'default_path_env_key': 'PATH',
                                           'detect_version': True,
                                           'env_path_key': 'CM_ARIA2_BIN_WITH_PATH',
                                           'run_script_input': i['run_script_input'],
                                           'recursion_spaces': recursion_spaces})
        if r['return'] > 0:
            if r['return'] == 16:
                # Not found, try install
                force_install = True
            else:
                return r

    # Force install
    if force_install:
        # Attempt to run installer
        version = env.get('CM_VERSION', '')
        if version == '':
            version = env['CM_ARIA2_DEFAULT_INSTALL_VERSION']

        if os_info['platform'] == 'windows':
            archive = 'aria2-{}-win-64bit-build1'
            ext = '.zip'
            ext2 = ''
        else:
            archive = 'aria2-{}'
            ext = '.tar.bz2'
            ext2 = '.tar'

        archive = archive.format(version)
        archive_with_ext = archive + ext

        env['CM_ARIA2_DOWNLOAD_DIR'] = archive

        env['CM_ARIA2_DOWNLOAD_FILE'] = archive_with_ext
        if ext2 != '':
            env['CM_ARIA2_DOWNLOAD_FILE2'] = archive + ext2

        url = 'https://github.com/aria2/aria2/releases/download/release-{}/{}'.format(
            version, archive_with_ext)
        env['CM_ARIA2_DOWNLOAD_URL'] = url

        print('URL to download ARIA2: {}'.format(url))

        r = automation.run_native_script(
            {'run_script_input': run_script_input, 'env': env, 'script_name': 'install'})
        if r['return'] > 0:
            return r

        if os_info['platform'] == 'windows' or env.get(
                'CM_ARIA2_BUILD_FROM_SRC', '').lower() == 'true':
            install_path = os.path.join(os.getcwd(), archive)

            path_to_file = os.path.join(install_path, file_name)
            if not os.path.isfile(path_to_file):
                return {'return': 1,
                        'error': 'file not found: {}'.format(path_to_file)}

            env['CM_ARIA2_BIN_WITH_PATH'] = path_to_file
            env['CM_ARIA2_INSTALLED_TO_CACHE'] = 'yes'
        else:
            path_to_bin = r['env_tmp'].get('CM_ARIA2_BIN_WITH_PATH', '')
            env['CM_ARIA2_BIN_WITH_PATH'] = path_to_bin

            r = i['automation'].find_artifact({'file_name': file_name,
                                               'env': env,
                                               'os_info': os_info,
                                               'default_path_env_key': 'PATH',
                                               'detect_version': True,
                                               'env_path_key': 'CM_ARIA2_BIN_WITH_PATH',
                                               'run_script_input': i['run_script_input'],
                                               'recursion_spaces': recursion_spaces})
            if r['return'] > 0:
                return r

    return {'return': 0}


def detect_version(i):
    env = i['env']

    r = i['automation'].parse_version({'match_text': r'aria2 version\s*([\d.]+)',
                                       'group_number': 1,
                                       'env_key': 'CM_ARIA2_VERSION',
                                       'which_env': i['env']})
    if r['return'] > 0:
        return r

    version = r['version']
    print(i['recursion_spaces'] + '    Detected version: {}'.format(version))

    return {'return': 0, 'version': version}


def postprocess(i):

    env = i['env']
    r = detect_version(i)
    if r['return'] > 0:
        return r

    version = r['version']
    found_file_path = env['CM_ARIA2_BIN_WITH_PATH']

    found_path = os.path.dirname(found_file_path)

    env['CM_ARIA2_INSTALLED_PATH'] = found_path

    if env.get('CM_ARIA2_INSTALLED_TO_CACHE', '') == 'yes':
        env['+PATH'] = [env['CM_ARIA2_INSTALLED_PATH']]

    return {'return': 0, 'version': version}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import cmind as cm


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    clean_cmd = ''
    cache_rm_tags = ''
    extra_cache_rm_tags = env.get('CM_CLEAN_EXTRA_CACHE_RM_TAGS', '')

    if env.get('CM_MODEL', '') == 'sdxl':
        if env.get('CM_CLEAN_ARTIFACT_NAME', '') == 'downloaded_data':
            clean_cmd = f"""rm -rf {os.path.join(env['CM_NVIDIA_MLPERF_SCRATCH_PATH'], "data", "coco", "SDXL")} """
            cache_rm_tags = "nvidia-harness,_preprocess_data,_sdxl"
        if env.get('CM_CLEAN_ARTIFACT_NAME', '') == 'preprocessed_data':
            clean_cmd = f"""rm -rf {os.path.join(env['CM_NVIDIA_MLPERF_SCRATCH_PATH'], "preprocessed_data", "coco2014-tokenized-sdxl")} """
            cache_rm_tags = "nvidia-harness,_preprocess_data,_sdxl"
        if env.get('CM_CLEAN_ARTIFACT_NAME', '') == 'downloaded_model':
            clean_cmd = f"""rm -rf {os.path.join(env['CM_NVIDIA_MLPERF_SCRATCH_PATH'], "models", "SDXL")} """
            cache_rm_tags = "nvidia-harness,_download_model,_sdxl"

    cache_rm_tags = cache_rm_tags + extra_cache_rm_tags

    if cache_rm_tags:
        r = cm.access({'action': 'rm', 'automation': 'cache',
                      'tags': cache_rm_tags, 'f': True})
        print(r)
        if r['return'] != 0 and r['return'] != 16:  # ignore missing ones
            return r
        if r['return'] == 0:  # cache entry found
            if clean_cmd != '':
                env['CM_RUN_CMD'] = clean_cmd
    else:
        if clean_cmd != '':
            env['CM_RUN_CMD'] = clean_cmd

    return {'return': 0}


def postprocess(i):

    env = i['env']

    return {'return': 0}



# Developer(s): Grigori Fursin

import cmind
import os
import datetime
import misc


def page(st, params):

    url_prefix = st.config.get_option('server.baseUrlPath') + '/'

    name = params.get('name', [''])[0].strip()
    tags = params.get('tags', [''])[0].lower()

    ii = {'action': 'find',
          'automation': 'report,6462ecdba2054467'}

    if name != '':
        ii['artifact'] = name
    if tags != '':
        ii['tags'] = tags

    r = cmind.access(ii)
    if r['return'] > 0:
        return r

    lst = r['list']

    end_html = ''

    ##########################################################################
    if len(lst) == 0:
        st.markdown('Reports were not found!')

    ##########################################################################
    elif len(lst) == 1:
        l = lst[0]

        meta = l.meta

        uid = meta['uid']

        title = meta.get('title', meta['alias'])

        path = l.path

        x = '''
            <center>
             <h3>Community report</h3>
             <h5>{}</h5>
            </center>
            '''.format(title)

        st.write(x, unsafe_allow_html=True)

        end_html = '<center><small><i><a href="{}">Self link</a></i></small></center>'.format(
            misc.make_url(meta['uid'], action='reports', md=False))

        # Check basic password
        password_hash = meta.get('password_hash', '')
        view = True
        if password_hash != '':
            view = False

            password = st.text_input(
                "Enter password",
                type="password",
                key="password")

            if password != '':
                import bcrypt
                # TBD: temporal hack to demo password protection for
                # experiments
                password_salt = b'$2b$12$ionIRWe5Ft7jkn4y/7C6/e'
                password_hash2 = bcrypt.hashpw(
                    password.encode('utf-8'), password_salt)

                if password_hash.encode('utf-8') == password_hash2:
                    view = True
                else:
                    st.markdown('**Warning:** wrong password')

        if not view:
            return {'return': 0, 'end_html': end_html}

        # Check if has text
        for f in ['README.md']:
            f1 = os.path.join(path, f)
            if os.path.isfile(f1):
                r = cmind.utils.load_txt(f1)
                if r['return'] > 0:
                    return r

                s = r['string']

                st.markdown('---')

                if f.endswith('.html'):
                    y = s.split('\n')
                    ss = ''
                    for x in y:
                        ss += x.strip() + '\n'

                    st.write(ss, unsafe_allow_html=True)
                else:
                    st.markdown(s)

                break

    ##########################################################################
    else:
        reports = []

        md = ''

        for l in sorted(lst, key=lambda x: x.meta.get(
                'date', ''), reverse=True):

            meta = l.meta

            if meta.get('private', False):
                continue

            uid = meta['uid']

            title = meta.get('title', meta['alias'])

            url = meta.get('redirect', '')
            if url == '':
                url = url_prefix + '?action=reports&name={}'.format(uid)

            md += '* [' + title + '](' + url + ')\n'

        x = '''
            <center>
             <h3>Community reports</h3>
            </center>
            '''
        st.write(x, unsafe_allow_html=True)

        st.markdown(md)

    return {'return': 0, 'end_html': end_html}



"""
mlperf inference benchmarking tool
"""

from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import argparse
import array
import collections
import json
import logging
import os
import sys
import threading
import time
from queue import Queue
from PIL import Image
import mlperf_loadgen as lg
import numpy as np
import cv2
import glob
import dataset
import cognata
import cognata_labels

# import imagenet
# import coco
# import openimages

logging.basicConfig(level=logging.INFO)
log = logging.getLogger("main")

NANO_SEC = 1e9
MILLI_SEC = 1000

# pylint: disable=missing-docstring

# the datasets we support
SUPPORTED_DATASETS = {
    "cognata-4mp-pt":
        (cognata.Cognata, None, cognata.PostProcessCognataPt(0.5, 200, 0.05, 1440, 2560),
         {"image_size": [1440, 2560, 3]}),
    "cognata-8mp-pt":
        (cognata.Cognata, None, cognata.PostProcessCognataPt(0.5, 200, 0.05, 2160, 3840),
         {"image_size": [2160, 3840, 3]})
}

# pre-defined command line options so simplify things. They are used as defaults and can be
# overwritten from command line

SUPPORTED_PROFILES = {
    "defaults": {
        "dataset": "imagenet",
        "backend": "tensorflow",
        "cache": 0,
        "max-batchsize": 32,
    },

    # retinanet
    "retinanet-pytorch": {
        "inputs": "image",
        "outputs": "boxes,labels,scores",
        "dataset": "openimages-800-retinanet",
        "backend": "pytorch-native",
        "model-name": "retinanet",
    },
}

SCENARIO_MAP = {
    "SingleStream": lg.TestScenario.SingleStream,
    "MultiStream": lg.TestScenario.MultiStream,
    "Server": lg.TestScenario.Server,
    "Offline": lg.TestScenario.Offline,
}

last_timeing = []


def get_args():
    """Parse commandline."""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--dataset",
        choices=SUPPORTED_DATASETS.keys(),
        help="dataset")
    parser.add_argument(
        "--dataset-path",
        required=True,
        help="path to the dataset")
    parser.add_argument("--dataset-list", help="path to the dataset list")
    parser.add_argument(
        "--data-format",
        choices=[
            "NCHW",
            "NHWC"],
        help="data format")
    parser.add_argument(
        "--profile",
        choices=SUPPORTED_PROFILES.keys(),
        help="standard profiles")
    parser.add_argument("--scenario", default="SingleStream",
                        help="mlperf benchmark scenario, one of " + str(list(SCENARIO_MAP.keys())))
    parser.add_argument(
        "--max-batchsize",
        type=int,
        help="max batch size in a single inference")
    parser.add_argument("--model", required=True, help="model file")
    parser.add_argument("--output", default="output", help="test results")
    parser.add_argument("--inputs", help="model inputs")
    parser.add_argument("--outputs", help="model outputs")
    parser.add_argument("--backend", help="runtime to use")
    parser.add_argument(
        "--model-name",
        help="name of the mlperf model, ie. resnet50")
    parser.add_argument(
        "--threads",
        default=os.cpu_count(),
        type=int,
        help="threads")
    parser.add_argument("--qps", type=int, help="target qps")
    parser.add_argument("--cache", type=int, default=0, help="use cache")
    parser.add_argument(
        "--cache_dir",
        type=str,
        default=None,
        help="dir path for caching")
    parser.add_argument(
        "--preprocessed_dir",
        type=str,
        default=None,
        help="dir path for storing preprocessed images (overrides cache_dir)")
    parser.add_argument(
        "--use_preprocessed_dataset",
        action="store_true",
        help="use preprocessed dataset instead of the original")
    parser.add_argument(
        "--accuracy",
        action="store_true",
        help="enable accuracy pass")
    parser.add_argument(
        "--find-peak-performance",
        action="store_true",
        help="enable finding peak performance pass")
    parser.add_argument(
        "--debug",
        action="store_true",
        help="debug, turn traces on")

    # file to use mlperf rules compliant parameters
    parser.add_argument(
        "--mlperf_conf",
        default="../../mlperf.conf",
        help="mlperf rules config")
    # file for user LoadGen settings such as target QPS
    parser.add_argument(
        "--user_conf",
        default="user.conf",
        help="user config for user LoadGen settings such as target QPS")
    # file for LoadGen audit settings
    parser.add_argument(
        "--audit_conf",
        default="audit.config",
        help="config for LoadGen audit settings")

    # below will override mlperf rules compliant settings - don't use for
    # official submission
    parser.add_argument("--time", type=int, help="time to scan in seconds")
    parser.add_argument("--count", type=int, help="dataset items to use")
    parser.add_argument(
        "--performance-sample-count",
        type=int,
        help="performance sample count")
    parser.add_argument(
        "--max-latency",
        type=float,
        help="mlperf max latency in pct tile")
    parser.add_argument(
        "--samples-per-query",
        default=8,
        type=int,
        help="mlperf multi-stream samples per query")
    args = parser.parse_args()

    # don't use defaults in argparser. Instead we default to a dict, override that with a profile
    # and take this as default unless command line give
    defaults = SUPPORTED_PROFILES["defaults"]

    if args.profile:
        profile = SUPPORTED_PROFILES[args.profile]
        defaults.update(profile)
    for k, v in defaults.items():
        kc = k.replace("-", "_")
        if getattr(args, kc) is None:
            setattr(args, kc, v)
    if args.inputs:
        args.inputs = args.inputs.split(",")
    if args.outputs:
        args.outputs = args.outputs.split(",")

    if args.scenario not in SCENARIO_MAP:
        parser.error("valid scanarios:" + str(list(SCENARIO_MAP.keys())))
    return args


def get_backend(backend):
    if backend == "null":
        from backend_null import BackendNull
        backend = BackendNull()
    elif backend == "pytorch":
        from backend_pytorch import BackendPytorch
        backend = BackendPytorch()
    elif backend == "pytorch-native":
        from backend_pytorch_native import BackendPytorchNative
        backend = BackendPytorchNative()
    else:
        raise ValueError("unknown backend: " + backend)
    return backend


class Item:
    """An item that we queue for processing by the thread pool."""

    def __init__(self, query_id, content_id, img, label=None):
        self.query_id = query_id
        self.content_id = content_id
        self.img = img
        self.label = label
        self.start = time.time()


class RunnerBase:
    def __init__(self, model, ds, threads, post_proc=None, max_batchsize=128):
        self.take_accuracy = False
        self.ds = ds
        self.model = model
        self.post_process = post_proc
        self.threads = threads
        self.take_accuracy = False
        self.max_batchsize = max_batchsize
        self.result_timing = []
        self.proc_results = []

    def handle_tasks(self, tasks_queue):
        pass

    def start_run(self, result_dict, take_accuracy):
        self.result_dict = result_dict
        self.result_timing = []
        self.take_accuracy = take_accuracy
        self.post_process.start()

    def run_one_item(self, qitem):
        # run the prediction
        processed_results = []
        try:
            results = self.model.predict({self.model.inputs[0]: qitem.img})

            processed_results = self.post_process(
                results, qitem.content_id, qitem.label, self.result_dict)
            if self.take_accuracy:
                self.post_process.add_results(processed_results)

            self.result_timing.append(time.time() - qitem.start)

        except Exception as ex:  # pylint: disable=broad-except
            src = [self.ds.get_item_loc(i) for i in qitem.content_id]
            log.error("thread: failed on contentid=%s, %s", src, ex)
            # since post_process will not run, fake empty responses
            processed_results = [[]] * len(qitem.query_id)
        finally:
            response_array_refs = []
            response = []
            for idx, query_id in enumerate(qitem.query_id):

                # Temporal hack for Cognata to add only boxes - fix
                processed_results2 = [x['boxes'].numpy()
                                      for x in processed_results[idx]]
                self.proc_results.append([{'boxes': x['boxes'].tolist(), 'scores': x['scores'].tolist(), 'labels': x['labels'].tolist(), 'id': x['id']}
                                          for x in processed_results[idx]])
                response_array = array.array("B", np.array(
                    processed_results2, np.float32).tobytes())
                response_array_refs.append(response_array)
                bi = response_array.buffer_info()
                response.append(lg.QuerySampleResponse(query_id, bi[0], bi[1]))
            lg.QuerySamplesComplete(response)

    def enqueue(self, query_samples):
        idx = [q.index for q in query_samples]
        query_id = [q.id for q in query_samples]
        if len(query_samples) < self.max_batchsize:
            data, label = self.ds.get_samples(idx)
            self.run_one_item(Item(query_id, idx, data, label))
        else:
            bs = self.max_batchsize
            for i in range(0, len(idx), bs):
                data, label = self.ds.get_samples(idx[i:i + bs])
                self.run_one_item(
                    Item(query_id[i:i + bs], idx[i:i + bs], data, label))

    def finish(self):
        pass


class QueueRunner(RunnerBase):
    def __init__(self, model, ds, threads, post_proc=None, max_batchsize=128):
        super().__init__(model, ds, threads, post_proc, max_batchsize)
        self.tasks = Queue(maxsize=threads * 4)
        self.workers = []
        self.result_dict = {}

        for _ in range(self.threads):
            worker = threading.Thread(
                target=self.handle_tasks, args=(
                    self.tasks,))
            worker.daemon = True
            self.workers.append(worker)
            worker.start()

    def handle_tasks(self, tasks_queue):
        """Worker thread."""
        while True:
            qitem = tasks_queue.get()
            if qitem is None:
                # None in the queue indicates the parent want us to exit
                tasks_queue.task_done()
                break
            self.run_one_item(qitem)
            tasks_queue.task_done()

    def enqueue(self, query_samples):
        idx = [q.index for q in query_samples]
        query_id = [q.id for q in query_samples]

        if len(query_samples) < self.max_batchsize:
            data, label = self.ds.get_samples(idx)
            self.tasks.put(Item(query_id, idx, data, label))
        else:
            bs = self.max_batchsize
            for i in range(0, len(idx), bs):
                ie = i + bs

                data, label = self.ds.get_samples(idx[i:ie])
                self.tasks.put(Item(query_id[i:ie], idx[i:ie], data, label))

    def finish(self):
        # exit all threads
        for _ in self.workers:
            self.tasks.put(None)
        for worker in self.workers:
            worker.join()


def add_results(final_results, name, result_dict,
                result_list, took, show_accuracy=False):
    percentiles = [50., 80., 90., 95., 99., 99.9]
    buckets = np.percentile(result_list, percentiles).tolist()
    buckets_str = ",".join(["{}:{:.4f}".format(p, b)
                           for p, b in zip(percentiles, buckets)])

    if result_dict["total"] == 0:
        result_dict["total"] = len(result_list)
    # this is what we record for each run
    result = {
        "took": took,
        "mean": np.mean(result_list),
        "percentiles": {str(k): v for k, v in zip(percentiles, buckets)},
        "qps": len(result_list) / took,
        "count": len(result_list),
        "good_items": result_dict["good"],
        "total_items": result_dict["total"],
    }
    acc_str = ""
    if show_accuracy:
        result["accuracy"] = 100. * result_dict["good"] / result_dict["total"]
        acc_str = ", acc={:.3f}%".format(result["accuracy"])
        if "mAP" in result_dict:
            result["mAP"] = 100. * result_dict["mAP"]
            acc_str += ", mAP={:.3f}%".format(result["mAP"])
            if os.environ.get('CM_COGNATA_ACCURACY_DUMP_FILE', '') != '':
                accuracy_file = os.environ['CM_COGNATA_ACCURACY_DUMP_FILE']
                with open(accuracy_file, "w") as f:
                    f.write("{:.3f}%".format(result["mAP"]))

        if "mAP_classes" in result_dict:
            result['mAP_per_classes'] = result_dict["mAP_classes"]
            acc_str += ", mAP_classes={}".format(result_dict["mAP_classes"])

    # add the result to the result dict
    final_results[name] = result

    # to stdout
    print("{} qps={:.2f}, mean={:.4f}, time={:.3f}{}, queries={}, tiles={}".format(
        name, result["qps"], result["mean"], took, acc_str,
        len(result_list), buckets_str))

    print('======================================================================')

#########################################################################


def main():
    print('======================================================================')

    global last_timeing
    args = get_args()

    log.info(args)

    # Find backend
    backend = get_backend(args.backend)

    # Load model to backend (Grigori moved here before dataset
    #   since we get various info about pre-processing from the model)

    print('')
    print('Loading model ...')
    print('')

    model = backend.load(args.model, inputs=args.inputs, outputs=args.outputs)

#    print (model.num_classes)
#    print (model.image_size)

    # --count applies to accuracy mode only and can be used to limit the number of images
    # for testing.
    count_override = False
    count = args.count
    if count:
        count_override = True

    # dataset to use
    wanted_dataset, pre_proc, post_proc, kwargs = SUPPORTED_DATASETS[args.dataset]
#    if args.use_preprocessed_dataset:
#        pre_proc=None

    print('')
    print('Loading dataset and preprocessing if needed ...')
    print('* Dataset path: {}'.format(args.dataset_path))
    print('* Preprocessed cache path: {}'.format(args.cache_dir))
    print('')

    ds = wanted_dataset(data_path=args.dataset_path,
                        image_list=args.dataset_list,
                        name=args.dataset,
                        pre_process=pre_proc,
                        use_cache=args.cache,
                        count=count,
                        cache_dir=args.cache_dir,
                        preprocessed_dir=args.preprocessed_dir,
                        threads=args.threads,
                        model_config=model.config,           # For ABTF
                        model_num_classes=model.num_classes,  # For ABTF
                        model_image_size=model.image_size,   # For ABTF
                        **kwargs)

    # For ABTF - maybe find cleaner way
    post_proc.encoder = ds.encoder

    final_results = {
        "runtime": model.name(),
        "version": model.version(),
        "time": int(time.time()),
        "args": vars(args),
        "cmdline": str(args),
    }

    mlperf_conf = os.path.abspath(args.mlperf_conf)
    if not os.path.exists(mlperf_conf):
        log.error("{} not found".format(mlperf_conf))
        sys.exit(1)

    user_conf = os.path.abspath(args.user_conf)
    if not os.path.exists(user_conf):
        log.error("{} not found".format(user_conf))
        sys.exit(1)

    audit_config = os.path.abspath(args.audit_conf)

    if args.output:
        output_dir = os.path.abspath(args.output)
        os.makedirs(output_dir, exist_ok=True)
        os.chdir(output_dir)

    #
    # make one pass over the dataset to validate accuracy
    #
    count = ds.get_item_count()

    # warmup
    if os.environ.get('CM_ABTF_ML_MODEL_SKIP_WARMUP',
                      '').strip().lower() != 'yes':
        ds.load_query_samples([0])
        for _ in range(5):
            img, _ = ds.get_samples([0])
            _ = backend.predict({backend.inputs[0]: img})
        ds.unload_query_samples(None)

    scenario = SCENARIO_MAP[args.scenario]
    runner_map = {
        lg.TestScenario.SingleStream: RunnerBase,
        lg.TestScenario.MultiStream: QueueRunner,
        lg.TestScenario.Server: QueueRunner,
        lg.TestScenario.Offline: QueueRunner
    }

    runner = runner_map[scenario](
        model,
        ds,
        args.threads,
        post_proc=post_proc,
        max_batchsize=args.max_batchsize)

    def issue_queries(query_samples):
        runner.enqueue(query_samples)

    def flush_queries():
        pass

    log_output_settings = lg.LogOutputSettings()
    log_output_settings.outdir = output_dir
    log_output_settings.copy_summary_to_stdout = False
    log_settings = lg.LogSettings()
    log_settings.enable_trace = args.debug
    log_settings.log_output = log_output_settings

    settings = lg.TestSettings()
    settings.FromConfig(mlperf_conf, args.model_name, args.scenario)
    settings.FromConfig(user_conf, args.model_name, args.scenario)
    settings.scenario = scenario
    settings.mode = lg.TestMode.PerformanceOnly
    if args.accuracy:
        settings.mode = lg.TestMode.AccuracyOnly
    if args.find_peak_performance:
        settings.mode = lg.TestMode.FindPeakPerformance

    if args.time:
        # override the time we want to run
        settings.min_duration_ms = args.time * MILLI_SEC
        settings.max_duration_ms = args.time * MILLI_SEC

    if args.qps:
        qps = float(args.qps)
        settings.server_target_qps = qps
        settings.offline_expected_qps = qps

    if count_override:
        settings.min_query_count = count
        settings.max_query_count = count

    if args.samples_per_query:
        settings.multi_stream_samples_per_query = args.samples_per_query

    if args.max_latency:
        settings.server_target_latency_ns = int(args.max_latency * NANO_SEC)
        settings.multi_stream_expected_latency_ns = int(
            args.max_latency * NANO_SEC)

    performance_sample_count = args.performance_sample_count if args.performance_sample_count else min(
        count, 500)
    sut = lg.ConstructSUT(issue_queries, flush_queries)
    qsl = lg.ConstructQSL(
        count,
        performance_sample_count,
        ds.load_query_samples,
        ds.unload_query_samples)

    log.info("starting {}".format(scenario))
    result_dict = {"good": 0, "total": 0, "scenario": str(scenario)}
    runner.start_run(result_dict, args.accuracy)

    lg.StartTestWithLogSettings(sut, qsl, settings, log_settings, audit_config)

    if not last_timeing:
        last_timeing = runner.result_timing
    if args.accuracy:
        post_proc.finalize(result_dict, ds, output_dir=args.output)

    add_results(final_results, "{}".format(scenario),
                result_dict, last_timeing, time.time() - ds.last_loaded, args.accuracy)

    runner.finish()
    lg.DestroyQSL(qsl)
    lg.DestroySUT(sut)
    #
    # write final results
    #
    if args.output:
        with open("results.json", "w") as f:
            json.dump(final_results, f, sort_keys=True, indent=4)
        if args.accuracy:
            print('Saving model output examples ...')
            files = glob.glob(
                os.path.join(
                    args.dataset_path,
                    '10002_Urban_Clear_Morning',
                    'Cognata_Camera_01_8M_png',
                    '*.png'))
            files = sorted(files)
            for pred_batch in runner.proc_results:
                for pred in pred_batch:
                    f = files[pred['id']]
                    cls_threshold = 0.3
                    img = Image.open(f).convert("RGB")
                    loc, label, prob = np.array(
                        pred['boxes']), np.array(
                        pred['labels']), np.array(
                        pred['scores'])
                    best = np.argwhere(prob > cls_threshold).squeeze(axis=1)

                    loc = loc[best]
                    label = label[best]
                    prob = prob[best]

                    # Update input image with boxes and predictions
                    output_img = cv2.imread(f)
                    if len(loc) > 0:

                        loc = loc.astype(np.int32)

                        for box, lb, pr in zip(loc, label, prob):
                            category = cognata_labels.label_info[lb]
                            color = cognata_labels.colors[lb]

                            xmin, ymin, xmax, ymax = box

                            cv2.rectangle(
                                output_img, (xmin, ymin), (xmax, ymax), color, 2)

                            text_size = cv2.getTextSize(
                                category + " : %.2f" %
                                pr, cv2.FONT_HERSHEY_PLAIN, 1, 1)[0]

                            cv2.rectangle(
                                output_img, (xmin, ymin), (xmin + text_size[0] + 3, ymin + text_size[1] + 4), color, -1)

                            cv2.putText(
                                output_img, category + " : %.2f" % pr,
                                (xmin, ymin +
                                 text_size[1] +
                                    4), cv2.FONT_HERSHEY_PLAIN, 1,
                                (255, 255, 255), 1)
                        output = "{}_prediction.jpg".format(f[:-4])

                        d1 = os.path.join(os.path.dirname(output), 'output')
                        if not os.path.isdir(d1):
                            os.makedirs(d1)

                        d2 = os.path.basename(output)

                        output = os.path.join(d1, d2)
                        cv2.imwrite(output, output_img)
            with open("preds.json", "w") as f:
                json.dump(runner.proc_results, f, indent=4)


if __name__ == "__main__":
    main()



# Taken from https://github.com/mlcommons/croissant/pull/564/files (@mkuchnik)

import os
import mlcroissant as mlc


def main():

    url = os.environ.get('CM_PRINT_CROISSANT_URL', '')

    if url == '':
        print('Error: --url is not specified')
        exit(1)

    ds = mlc.Dataset(url)
    metadata = ds.metadata.to_json()

    print('')
    print('Croissant meta data URL: {}'.format(url))
    print('')
    print(f"{metadata['name']}: {metadata['description']}")

    print('')
    for x in ds.records(record_set="default"):
        print(x)


if __name__ == '__main__':
    main()



# Developer(s): Grigori Fursin

import streamlit as st
from streamlit.components.v1 import html
from streamlit_option_menu import option_menu

import os
import cmind
import misc


def main():

    st.set_page_config(layout="wide",
                       menu_items={})

    params = misc.get_params(st)

    # Set style
    # Green: background:#7fcf6f;
    hide_streamlit_style = """
            <style>
            #MainMenu {visibility: hidden;}

            button {
                background:#2f6fb3;
                border:0 none;
                -webkit-border-radius: 2px;
                border-radius: 2px;
                color: #ffffff;
                margin:3px;
            }

            </style>
            """

    st.markdown(hide_streamlit_style, unsafe_allow_html=True)

    # Set title (check extra user HTML to embed before title if needed)
    extra = os.environ.get('CM_GUI_EXTRA_HTML', '')

    if extra != '':
        url = ''
        for p in params:
            v = str(','.join(params[p]))
            if url != '':
                url += ';'
            url += p + '=' + v
        extra = extra.replace('{{CM_URL}}', url) + '\n\n'

    st.write('''
        <center>
        <h2 style="color:#2f6fb3">Collective Knowledge Playground</h2>
        <a href="/"><img src="https://cknowledge.org/images/logo-ck-tr.png" width="150"></a><br>
        {}
        <br>
        </center>
        '''.format(extra),
             unsafe_allow_html=True
             )

    extra_file = os.environ.get('CM_GUI_EXTRA_HTML_FILE', '')
    if extra_file != '':
        r = cmind.utils.load_txt(extra_file)
        if r['return'] > 0:
            return r

        s = '\n\n' + r['string'] + '\n\n'

        st.write(s, unsafe_allow_html=True)

    # Check action and basic menu
    action = params.get('action', ['scripts'])[0].lower()

    style_action_scripts = 'font-style:italic;font-weight:bold;color:#ffffff' if action == 'scripts' else ''
    style_action_howtorun = 'font-style:italic;font-weight:bold;color:#ffffff' if action == 'howtorun' else ''
    style_action_challenges = 'font-style:italic;font-weight:bold;color:#ffffff' if action == 'challenges' else ''
    style_action_contributors = 'font-style:italic;font-weight:bold;color:#ffffff' if action == 'contributors' else ''
    style_action_experiments = 'font-style:italic;font-weight:bold;color:#ffffff' if action == 'experiments' else ''
    style_action_reproduce = 'font-style:italic;font-weight:bold;color:#ffffff' if action == 'reproduce' else ''
    style_action_apps = 'font-style:italic;font-weight:bold;color:#ffffff' if action == 'apps' else ''
    style_action_reports = 'font-style:italic;font-weight:bold;color:#ffffff' if action == 'reports' else ''
    style_action_beta = 'font-style:italic;font-weight:bold;color:#ffffff' if action == 'beta' else ''
    style_action_install = 'font-style:italic;font-weight:bold;color:#ffffff' if action == 'install' else ''

    st.write('''
        <center>
        <a target="_self" href="?action=scripts"><button style="{}">Automation recipes</button></a>
        <a target="_self" href="?action=howtorun"><button style="{}">Modular benchmarks</button></a>
        <a target="_self" href="?action=challenges"><button style="{}">Challenges</button></a>
        <a target="_self" href="?action=experiments"><button style="{}">Results</button></a>
        <a target="_self" href="?action=reproduce"><button style="{}">Reproducibility</button></a><br>
        <a target="_self" href="?action=contributors"><button style="{}">Leaderboard</button></a>
        <a target="_self" href="?action=reports"><button style="{}">Reports</button></a>
        <a target="_self" href="?action=beta"><button style="{}">Beta</button></a>
        <a target="_self" href="?action=scripts&tags=modular,app"><button style="{}">Modular apps</button></a><br>
        <a target="_self" href="https://discord.gg/JjWNWXKxwT"><button>Discord</button></a>
        <a target="_self" href="https://github.com/mlcommons/ck"><button>GitHub</button></a>
        <a target="_self" href="?action=install"><button style="{}">Install</button></a>
        </center>
        '''.format(
        style_action_scripts,
        style_action_howtorun,
        style_action_challenges,
        style_action_experiments,
        style_action_reproduce,
        style_action_contributors,
        style_action_reports,
        style_action_beta,
        style_action_apps,
        style_action_install
    ),
        unsafe_allow_html=True
    )

    # Check actions
#    st.markdown("""---""")
    st.markdown('')

    r = {'return': 0}

    if action == 'challenges':
        from playground_challenges import page
        r = page(st, params)
    elif action == 'howtorun':
        from playground_howtorun import page
        r = page(st, params)
    elif action == 'experiments':
        from graph import visualize
        r = visualize(st, params, action='experiments')
    elif action == 'contributors':
        from playground_contributors import page
        r = page(st, params)
    elif action == 'scripts' or action == 'recipes' or action == 'automation-recipes' or action == 'components':
        from playground_scripts import page
        r = page(st, params)
    elif action == 'reproduce' or action == 'repro' or action == 'reproducibility':
        from playground_reproduce import page
        r = page(st, params)
    elif action == 'apps' or action == 'optimized-apps':
        from playground_apps import page
        r = page(st, params)
    elif action == 'reports':
        from playground_reports import page
        r = page(st, params)
    elif action == 'beta':
        from playground_beta import page
        r = page(st, params)
    elif action == 'install' or action == 'setup':
        from playground_install import page
        r = page(st, params, {})

    if r['return'] > 0:
        st.markdown(
            '**CM error:** {} . Please report [here](https://github.com/mlcommons/ck/issues)'.format(r['error']))

    end_html = r.get('end_html', '')

    # Finalize all pages
    st.markdown("""---""")

    if end_html != '':
        st.write(end_html, unsafe_allow_html=True)

    st.write("""
             <center>
              Powered by <a href="https://github.com/mlcommons/ck">MLCommons Collective Mind</a>
             </center>
             """,
             unsafe_allow_html=True)


def make_url(name, alias='', action='contributors', key='name', md=True):

    import urllib

    if alias == '':
        alias = name

    url = '?action={}&{}={}'.format(
        action, key, urllib.parse.quote_plus(alias))

    if md:
        md = '[{}]({})'.format(name, url)
    else:
        md = url

    return md


def convert_date(date):
    # date: format YYYYMMDD to YYYY month day

    import calendar

    try:
        year = date[0:4]
        month = calendar.month_abbr[int(date[4:6])]
        day = str(int(date[6:8]))
    except Exception as e:
        return {'return': 1, 'error': 'date "{}" is not of format YYYYMMDD: {}'.format(
            date, format(e))}

    return {'return': 0, 'string': year + ' ' + month + ' ' + day}


if __name__ == "__main__":
    main()



# Author: Grigori Fursin
# Contributors: Arjun Suresh, Anandhu Sooraj
#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

import os
from cmind import utils

# Pring help about script


def print_help(i):

    meta = i.get('meta', '')
    path = i.get('path', '')

    if len(meta) == 0 and path == '':
        return {'return': 0}

    print('')
    print(
        'Help for this CM script ({},{}):'.format(
            meta.get(
                'alias', ''), meta.get(
                'uid', '')))

    print('')
    print('Path to this automation recipe: {}'.format(path))

    variations = meta.get('variations', {})
    if len(variations) > 0:
        print('')
        print('Available variations:')
        print('')
        for v in sorted(variations):
            print('  _' + v)

    input_mapping = meta.get('input_mapping', {})
    if len(input_mapping) > 0:
        print('')
        print('Available flags mapped to environment variables:')
        print('')
        for k in sorted(input_mapping):
            v = input_mapping[k]

            print('  --{}  ->  --env.{}'.format(k, v))

    input_description = meta.get('input_description', {})
    if len(input_description) > 0:
        # Check if has important ones (sort)
        sorted_keys = []
        all_keys = sorted(list(input_description.keys()))

        for k in sorted(
                all_keys, key=lambda x: input_description[x].get('sort', 0)):
            v = input_description[k]
            if v.get('sort', 0) > 0:
                sorted_keys.append(k)

        print('')
        print('Available flags (Python API dict keys):')
        print('')
        for k in all_keys:
            v = input_description[k]
            n = v.get('desc', '')

            x = '  --' + k
            if n != '':
                x += '  ({})'.format(n)

            print(x)

        if len(sorted_keys) > 0:
            print('')
            print('Main flags:')
            print('')
            for k in sorted_keys:
                v = input_description[k]
                n = v.get('desc', '')

                x = '  --' + k

                d = None
                if 'default' in v:
                    d = v.get('default', '')

                if d is not None:
                    x += '=' + d

                c = v.get('choices', [])
                if len(c) > 0:
                    x += '   {' + ','.join(c) + '}'

                if n != '':
                    x += '   ({})'.format(n)

                print(x)

    print('')
    x = input('Would you like to see a Python API with a list of common keys/flags for all scripts including this one (y/N)? ')

    x = x.strip().lower()

    skip_delayed_help = False if x in ['y', 'yes'] else True

    r = {'return': 0}

    if skip_delayed_help:
        r['skip_delayed_help'] = True

    return r



#!/usr/bin/env python3

import os
import numpy as np


# Processing in batches:
#
BATCH_SIZE = int(os.getenv('CM_BATCH_SIZE', 1))


# Model properties:
#
MODEL_IMAGE_HEIGHT = int(os.getenv('CM_ML_MODEL_IMAGE_HEIGHT',
                                   os.getenv('CM_ONNX_MODEL_IMAGE_HEIGHT',
                                             os.getenv('CM_TENSORFLOW_MODEL_IMAGE_HEIGHT',
                                                       ''))))
MODEL_IMAGE_WIDTH = int(os.getenv('CM_ML_MODEL_IMAGE_WIDTH',
                                  os.getenv('CM_ONNX_MODEL_IMAGE_WIDTH',
                                            os.getenv('CM_TENSORFLOW_MODEL_IMAGE_WIDTH',
                                                      ''))))
MODEL_IMAGE_CHANNELS = int(os.getenv('CM_ML_MODEL_IMAGE_CHANNELS', 3))
MODEL_DATA_LAYOUT = os.getenv('CM_ML_MODEL_DATA_LAYOUT', 'NCHW')
MODEL_COLOURS_BGR = os.getenv(
    'CM_ML_MODEL_COLOUR_CHANNELS_BGR', 'NO') in (
        'YES', 'yes', 'ON', 'on', '1')
MODEL_INPUT_DATA_TYPE = os.getenv('CM_ML_MODEL_INPUT_DATA_TYPE', 'float32')
MODEL_DATA_TYPE = os.getenv('CM_ML_MODEL_DATA_TYPE', '(unknown)')
MODEL_USE_DLA = os.getenv(
    'CM_ML_MODEL_USE_DLA',
    'NO') in (
        'YES',
        'yes',
        'ON',
        'on',
    '1')
MODEL_MAX_BATCH_SIZE = int(os.getenv('CM_ML_MODEL_MAX_BATCH_SIZE', BATCH_SIZE))


# Internal processing:
#
INTERMEDIATE_DATA_TYPE = np.float32    # default for internal conversion
# INTERMEDIATE_DATA_TYPE  = np.int8       # affects the accuracy a bit


# Image normalization:
#
MODEL_NORMALIZE_DATA = os.getenv('CM_ML_MODEL_NORMALIZE_DATA') in (
    'YES', 'yes', 'ON', 'on', '1')
MODEL_NORMALIZE_LOWER = float(os.getenv('CM_ML_MODEL_NORMALIZE_LOWER', -1.0))
MODEL_NORMALIZE_UPPER = float(os.getenv('CM_ML_MODEL_NORMALIZE_UPPER', 1.0))
SUBTRACT_MEAN = os.getenv(
    'CM_ML_MODEL_SUBTRACT_MEANS', 'YES') in (
        'YES', 'yes', 'ON', 'on', '1')
GIVEN_CHANNEL_MEANS = os.getenv('CM_ML_MODEL_GIVEN_CHANNEL_MEANS', '')
if GIVEN_CHANNEL_MEANS:
    GIVEN_CHANNEL_MEANS = np.fromstring(
        GIVEN_CHANNEL_MEANS,
        dtype=np.float32,
        sep=' ').astype(INTERMEDIATE_DATA_TYPE)
    if MODEL_COLOURS_BGR:
        # swapping Red and Blue colour channels
        GIVEN_CHANNEL_MEANS = GIVEN_CHANNEL_MEANS[::-1]

GIVEN_CHANNEL_STDS = os.getenv('CM_ML_MODEL_GIVEN_CHANNEL_STDS', '')
if GIVEN_CHANNEL_STDS:
    GIVEN_CHANNEL_STDS = np.fromstring(
        GIVEN_CHANNEL_STDS,
        dtype=np.float32,
        sep=' ').astype(INTERMEDIATE_DATA_TYPE)
    if MODEL_COLOURS_BGR:
        # swapping Red and Blue colour channels
        GIVEN_CHANNEL_STDS = GIVEN_CHANNEL_STDS[::-1]


# ImageNet dataset properties:
#
LABELS_PATH = os.environ['CM_CAFFE_IMAGENET_SYNSET_WORDS_TXT']


# Preprocessed input images' properties:
#
IMAGE_DIR = os.getenv('CM_DATASET_PREPROCESSED_PATH')
IMAGE_DATA_TYPE = os.getenv('CM_DATASET_PREPROCESSED_DATA_TYPE', 'float32')


def load_labels(labels_filepath):
    my_labels = []
    input_file = open(labels_filepath, 'r')
    for l in input_file:
        my_labels.append(l.strip())
    return my_labels


class_labels = load_labels(LABELS_PATH)


# Load preprocessed image filenames:
image_list = []
all_images = os.listdir(IMAGE_DIR)
for image_file in all_images:
    if image_file.endswith('.npy'):
        image_list.append(image_file)


def load_image_by_index_and_normalize(image_index):

    img_file = os.path.join(IMAGE_DIR, image_list[image_index])

    img = np.fromfile(img_file, np.dtype(IMAGE_DATA_TYPE))
    # img = img.reshape((1,MODEL_IMAGE_HEIGHT, MODEL_IMAGE_WIDTH, 3))
    img.resize(224 * 224 * 3)
    img = img.reshape(
        (MODEL_IMAGE_HEIGHT,
         MODEL_IMAGE_WIDTH,
         MODEL_IMAGE_CHANNELS))
    if MODEL_COLOURS_BGR:
        img = img[..., ::-1]     # swapping Red and Blue colour channels

    if IMAGE_DATA_TYPE != 'float32':
        img = img.astype(np.float32)

        # Normalize
        if MODEL_NORMALIZE_DATA:
            img /= (255.0 / (MODEL_NORMALIZE_UPPER - MODEL_NORMALIZE_LOWER))
            img += MODEL_NORMALIZE_LOWER

        # Subtract mean value
        if len(GIVEN_CHANNEL_MEANS):
            img -= GIVEN_CHANNEL_MEANS
        elif SUBTRACT_MEAN:
            img -= np.mean(img, axis=(0, 1), keepdims=True)

        if len(GIVEN_CHANNEL_STDS):
            img /= GIVEN_CHANNEL_STDS

    if MODEL_INPUT_DATA_TYPE == 'int8' or INTERMEDIATE_DATA_TYPE == np.int8:
        img = np.clip(img, -128, 127).astype(INTERMEDIATE_DATA_TYPE)

    if MODEL_DATA_LAYOUT == 'NCHW':
        img = img.transpose(2, 0, 1)
    elif MODEL_DATA_LAYOUT == 'CHW4':
        img = np.pad(img, ((0, 0), (0, 0), (0, 1)), 'constant')

    # Add img to batch
    return img.astype(MODEL_INPUT_DATA_TYPE)


def load_preprocessed_batch(image_list, image_index):
    batch_data = None
    for in_batch_idx in range(BATCH_SIZE):
        img = load_image_by_index_and_normalize(image_index)
        if batch_data is None:
            batch_data = np.empty(
                (BATCH_SIZE, *img.shape), dtype=MODEL_INPUT_DATA_TYPE)
        batch_data[in_batch_idx] = img
        image_index += 1

    # print('Data shape: {}'.format(batch_data.shape))

    if MODEL_USE_DLA and MODEL_MAX_BATCH_SIZE > len(batch_data):
        return np.pad(batch_data, ((0, MODEL_MAX_BATCH_SIZE - len(batch_data)),
                      (0, 0), (0, 0), (0, 0)), 'constant'), image_index
    else:
        return batch_data, image_index



# Developer: Grigori Fursin

import os


def main():
    # For now quick prototype hardwired to "summary.json" from MLPerf
    # Later need to clean it and make it universal

    print('')
    print('Reading summary.json ...')
    print('')

    import json
    filename = os.environ.get('MLPERF_INFERENCE_SUBMISSION_SUMMARY', '')
    if filename == '':
        filename = 'summary'
    filename += '.json'

    f = open(filename)

    results = json.load(f)

    f.close()

    print('=========================================================')
    print('Sending results to W&B dashboard ...')
    print('')

    import wandb

    env = os.environ

    dashboard_user = env.get('CM_MLPERF_DASHBOARD_WANDB_USER', '')
    if dashboard_user == '':
        dashboard_user = 'cmind'

    dashboard_project = env.get('CM_MLPERF_DASHBOARD_WANDB_PROJECT', '')
    if dashboard_project == '':
        dashboard_project = 'cm-mlperf-dse-testing'

    for k in results:

        result = results[k]

        organization = str(result.get('Organization', ''))
        if organization == '':
            organization = 'anonymous'

        label = organization

        system_name = str(result.get('SystemName', ''))
        if system_name != '':
            label += '(' + system_name + ')'

        qps = result.get('Result', 0.0)
        # since v4.1 mlperf results return a key:value pairs for accuracy. We
        # are taking only the first key:value here
        result_acc = result.get('Accuracy')
        accuracy = 0.0
        if result_acc:
            acc_split = result_acc.split(":")
            if len(acc_split) > 1:
                accuracy = float(acc_split[1]) / 100

        result['performance'] = qps
        result['qps'] = qps
        result['accuracy'] = accuracy

        # Check extra env variables
        x = {
            "lang": "CM_MLPERF_LANG",
            "device": "CM_MLPERF_DEVICE",
            "submitter": "CM_MLPERF_SUBMITTER",
            "backend": "CM_MLPERF_BACKEND",
            "model": "CM_MLPERF_MODEL",
            "run_style": "CM_MLPERF_RUN_STYLE",
            "rerun": "CM_RERUN",
            "hw_name": "CM_HW_NAME",
            "max_batchsize": "CM_MLPERF_LOADGEN_MAX_BATCHSIZE",
            "num_threads": "CM_NUM_THREADS",
            "scenario": "CM_MLPERF_LOADGEN_SCENARIO",
            "test_query_count": "CM_TEST_QUERY_COUNT",
            "run_checker": "CM_RUN_SUBMISSION_CHECKER",
            "skip_truncation": "CM_SKIP_TRUNCATE_ACCURACY"
        }

        for k in x:
            env_key = x[k]
            if os.environ.get(env_key, '') != '':
                result['cm_misc_input_' + k] = os.environ[env_key]

        wandb.init(entity=dashboard_user,
                   project=dashboard_project,
                   name=label)

        wandb.log(result)

        wandb.finish()

    print('=========================================================')


if __name__ == '__main__':
    main()



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']
    state = i['state']
    meta = i['meta']
    script_path = i['run_script_input']['path']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    models = env['MODELS'].split(",")

    backends = env.get('BACKENDS')
    if backends:
        backends = backends.split(",")

    devices = env.get('DEVICES')
    if devices:
        devices = devices.split(",")

    print(backends)
    implementation = env['IMPLEMENTATION']

    power = env.get('POWER', '')

    if str(power).lower() in ["yes", "true"]:
        POWER_STRING = " --power yes --adr.mlperf-power-client.power_server=" + \
            env.get('POWER_SERVER', '192.168.0.15') + " --adr.mlperf-power-client.port=" + \
            env.get('POWER_SERVER_PORT', '4950') + " "
    else:
        POWER_STRING = ""

    if not devices:
        return {
            'return': 1, 'error': 'No device specified. Please set one or more (comma separated) of {cpu, qaic, cuda, rocm} for --env.DEVICES=<>'}

    for model in models:
        env['MODEL'] = model
        cmds = []
        run_script_content = '#!/bin/bash\nsource ' + \
            os.path.join(script_path, "run-template.sh")

        if not backends:
            if implementation == "reference":
                if model == "resnet50":
                    backends = "tf,onnxruntime"
                elif model == "retinanet":
                    backends = "onnxruntime,pytorch"
                elif "bert" in model:
                    backends = "tf,onnxruntime,pytorch"
                elif "3d-unet" in model:
                    backends = "tf,onnxruntime,pytorch"
                elif model == "rnnt":
                    backends = "pytorch"
                elif "gptj" in model:
                    backends = "pytorch"
                elif "stable-diffusion-xl" in model:
                    backends = "pytorch"
                elif "llama2-70b" in model:
                    backends = "pytorch"
            backends = backends.split(",")

        for backend in backends:

            for device in devices:
                offline_target_qps = (
                    ((state.get(
                        model,
                        {})).get(
                        device,
                        {})).get(
                        backend,
                        {})).get('offline_target_qps')
                if offline_target_qps:
                    pass
                else:  # try to do a test run with reasonable number of samples to get and record the actual system performance
                    if device == "cpu":
                        if model == "resnet50":
                            test_query_count = 1000
                        else:
                            test_query_count = 100
                    else:
                        if model == "resnet50":
                            test_query_count = 10000
                        else:
                            test_query_count = 1000
                    cmd = f'run_test "{backend}" "{test_query_count}" "{implementation}" "{device}" "$find_performance_cmd"'
                    cmds.append(cmd)
                    # second argument is unused for submission_cmd
                cmd = f'run_test "{backend}" "100" "{implementation}" "{device}" "$submission_cmd"'
                cmds.append(cmd)
                run_file_name = 'tmp-' + model + '-run'
                run_script_content += "\n\n" + "\n\n".join(cmds)
                with open(os.path.join(script_path, run_file_name + ".sh"), 'w') as f:
                    f.write(run_script_content)
        print(cmds)

    return {'return': 0}


def postprocess(i):

    env = i['env']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    if env.get('CM_ML_MODEL_TF_FIX_INPUT_SHAPE', '') == "yes":
        i['run_script_input']['script_name'] = "run-fix-input"

    return {'return': 0}


def postprocess(i):

    env = i['env']

    if env.get('CM_ML_MODEL_TF_FIX_INPUT_SHAPE', '') == "yes":
        env['CM_ML_MODEL_STARTING_FILE_PATH'] = env['CM_ML_MODEL_FILE_WITH_PATH']
        env['CM_ML_MODEL_FILE_WITH_PATH'] = os.path.join(
            os.getcwd(), "resnet50_v1.pb")

    env['CM_ML_MODEL_FILE'] = os.path.basename(
        env['CM_ML_MODEL_FILE_WITH_PATH'])
    env['CM_GET_DEPENDENT_CACHED_PATH'] = env['CM_ML_MODEL_FILE_WITH_PATH']

    env['CM_DOWNLOAD_PATH'] = os.path.dirname(
        env['CM_ML_MODEL_FILE_WITH_PATH'])

    return {'return': 0}
